﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using CDBurnerXP.IO;
using CDBurnerXP.Controls;
using System.Xml;
using CDBurnerXP.IO.Cue;

namespace CDBurnerXP.Burning
{
    public class AudioCompilation : Compilation
    {
        private List<AudioTrack> m_AudioTracks = new List<AudioTrack>();
        private FileStream m_CurrentStream = null;
        private string m_Title = string.Empty;
        private string m_Artist = string.Empty;

        #region Properties

        /// <summary>
        /// Gets a list of all audio tracks.
        /// </summary>
        public List<AudioTrack> AudioTracks
        {
            get { return m_AudioTracks; }
        }

        /// <summary>
        /// Gets or sets the global CD-Text Title information.
        /// </summary>
        public string Title
        {
            get { return m_Title; }
            set { m_Title = value; }
        }

        /// <summary>
        /// Gets or sets the global CD-Text Title information.
        /// </summary>
        public string Artist
        {
            get { return m_Artist; }
            set { m_Artist = value; }
        }

        #endregion

        public AudioCompilation()
        {
        }

        public AudioCompilation(AudioTrack[] tracks)
        {
            m_AudioTracks = new List<AudioTrack>(tracks);
        }

        public override ItemAddError[]  Load(string filename)
        {
            m_LastProgress = -1;

            List<ItemAddError> errors = new List<ItemAddError>();

            XmlReaderSettings settings = new XmlReaderSettings();
            settings.ProhibitDtd = false;
            settings.ValidationType = ValidationType.None;

            using (m_CurrentStream = new FileStream(filename, FileMode.Open, FileAccess.Read))
            {
                XmlReader reader = XmlReader.Create(m_CurrentStream, settings);

                while (reader.Read())
                {
                    if (m_Cancel) break;

                    // Reading tracks is rather simple, we just have one element type
                    // that needs to be of our concern.
                    if (reader.NodeType != XmlNodeType.Element) continue;

                    switch (reader.Name)
                    {
                        case "track":
                        case "TRACK":
                            OnLoadProgressChanged(m_CurrentStream.Position, m_CurrentStream.Length);

                            string trackLocation = reader.GetAttribute("path");
                            if (trackLocation == null)
                            {
                                // 3.5 compatibility
                                trackLocation = reader.GetAttribute("Path");
                            }

                            try
                            {
                                AudioTrack track = new AudioTrack(trackLocation);
                                track.CDTextTitle = reader.GetAttribute("title");
                                track.CDTextArtist = reader.GetAttribute("artist");
                                m_AudioTracks.Add(track);
                            }
                            catch (Exception ex)
                            {
                                errors.Add(new ItemAddError(trackLocation, ex));
                            }
                            break;

                        case "compilation":
                            m_Artist = reader.GetAttribute("artist");
                            m_Title = reader.GetAttribute("title");
                            break;
                    }
                }
            }

            m_Errors = errors.ToArray();

            return errors.ToArray();
        }

        public override void Save(string filename)
        {
            using (FileStream stream = new FileStream(filename, FileMode.Create, FileAccess.Write, FileShare.None))
            {
                XmlWriterSettings settings = new XmlWriterSettings();
                settings.Indent = true;
                using (XmlWriter writer = XmlWriter.Create(stream, settings))
                {

                    // Doctype + Stylesheet
                    writer.WriteStartDocument(true);
                    writer.WriteDocType("layout", "http://www.cdburnerxp.se/help/audio.dtd", null, null);
                    writer.WriteProcessingInstruction("xml-stylesheet", "type='text/xsl' href='http://www.cdburnerxp.se/help/compilation.xsl'");
                    // Comment
                    string comment = "audio compilation created by CDBurnerXP " + Application.ProductVersion + " (http://cdburnerxp.se)";
                    writer.WriteComment(comment);
                    // Header
                    writer.WriteStartElement("layout");
                    writer.WriteAttributeString("type", "Audio");
                    writer.WriteAttributeString("version", Application.ProductVersion);
                    writer.WriteAttributeString("date", DateTime.Now.ToShortDateString());
                    writer.WriteAttributeString("time", DateTime.Now.ToShortTimeString());

                    writer.WriteStartElement("compilation");
                    writer.WriteAttributeString("name", "audio-template");
                    writer.WriteAttributeString("title", m_Title);
                    writer.WriteAttributeString("artist", m_Artist);

                    // Tracks
                    int counter = 1;
                    foreach (AudioTrack track in m_AudioTracks)
                    {
                        writer.WriteStartElement("track");

                        string path = track.FileName;
                        // Is this a ripped file? If so, we should try to rescue it out of the
                        // temp folder for later use.
                        if (track.FileName.IndexOf(PathEx.GetTempRipPath()) == 0)
                        {
                            try
                            {
                                string newName = Path.Combine(Path.GetDirectoryName(filename), Path.GetFileName(track.FileName));
                                File.Copy(track.FileName, newName);
                                path = newName;
                            }
                            catch (Exception)
                            {
                                // if failed, ignore
                            }
                        }
                        writer.WriteAttributeString("path", path);
                        writer.WriteAttributeString("title", track.CDTextTitle);
                        writer.WriteAttributeString("artist", track.CDTextArtist);
                        writer.WriteAttributeString("number", (counter++).ToString());
                        writer.WriteEndElement();
                        OnSaveProgressChanged(counter - 1, m_AudioTracks.Count);
                    }

                    // compilation
                    writer.WriteEndElement();

                    // layout
                    writer.WriteEndElement();
                }
            }
        }

        /// <summary>
        /// Saves the audio compilation as cue sheet.
        /// </summary>
        public void SaveAsCue(string filename)
        {
            CueSheet cueSheet = new CueSheet();
            foreach (AudioTrack track in m_AudioTracks)
            {
                cueSheet.AddTrack(track.Title, track.Artist, track.FileName, FileType.WAVE);
                cueSheet.Tracks[cueSheet.Tracks.Length - 1].AddIndex(1, 0, 0, 0);
            }
            cueSheet.Performer = this.Artist;
            cueSheet.Title = this.Title;
            cueSheet.SaveCue(filename, Encoding.UTF8);
        }

        /// <summary>
        /// Saves the audio compilation as M3U playlistt.
        /// </summary>
        public void SaveAsM3u(string filename)
        {
            using (FileStream stream = new FileStream(filename, FileMode.Create, FileAccess.Write, FileShare.None))
            {
                using (StreamWriter writer = new StreamWriter(stream))
                {
                    writer.WriteLine("#EXTM3U");

                    foreach (AudioTrack track in m_AudioTracks)
                    {
                        writer.WriteLine("#EXTINF:{0},{1} - {2}", track.Length.TotalSeconds.ToString("0"), track.Artist, track.Title);
                        writer.WriteLine(track.FileName);
                    }
                }
            }
        }
    }
}
