﻿using System;
using System.Collections.Generic;
using System.Text;
using CDBurnerXP.Controls;

namespace CDBurnerXP.Burning
{
    /// <summary>
    /// Base class for CDBurnerXP compilation files.
    /// Provides some basic functionality like progress events.
    /// </summary>
    public abstract class Compilation
    {
        protected int m_LastProgress = -1;
        protected bool m_Cancel = false;
        protected ItemAddError[] m_Errors = new ItemAddError[0];

        #region Events

        /// <summary>
        /// This event is fired, whenever the save progress of a compilation changes.
        /// </summary>
        public event EventHandler<CompilationProgressEventArgs> SaveProgressChanged;

        /// <summary>
        /// This event is fired, whenever the load progress of a compilation changes.
        /// </summary>
        public event EventHandler<CompilationProgressEventArgs> LoadProgressChanged;

        protected virtual void OnSaveProgressChanged(long pos, long length)
        {
            double progress = (double)pos / length * 100.0;
            byte progressInt = Convert.ToByte(Math.Round(progress));

            if (progressInt != m_LastProgress)
            {
                if (SaveProgressChanged != null)
                {
                    SaveProgressChanged(this, new CompilationProgressEventArgs(progressInt));
                }
                m_LastProgress = progressInt;
            }
        }

        protected virtual void OnLoadProgressChanged(long pos, long length)
        {
            double progress = (double)pos / length * 100.0;
            byte progressInt = Convert.ToByte(Math.Round(progress));

            if (progressInt != m_LastProgress)
            {
                if (LoadProgressChanged != null)
                {
                    LoadProgressChanged(this, new CompilationProgressEventArgs(progressInt));
                }
                m_LastProgress = progressInt;
            }
        }

        #endregion

        #region Properties

        /// <summary>
        /// Provides a list of errors which happened
        /// during the load progress (missing files etc).
        /// </summary>
        public ItemAddError[] Errors
        {
            get
            {
                return m_Errors;
            }
        }

        #endregion

        /// <summary>
        /// Loads a compilation from the given file name.
        /// </summary>
        /// <param name="filename">Location of the file</param>
        /// <returns>All errors that occured during loading</returns>
        abstract public ItemAddError[] Load(string filename);

        /// <summary>
        /// Saves a compilation to disc using the given file name.
        /// </summary>
        /// <param name="filename">Location of the file</param>
        abstract public void Save(string filename);

        /// <summary>
        /// Cancels the current loading or saving operation.
        /// </summary>
        public virtual void Cancel()
        {
            m_Cancel = true;
        }
    }
}
