using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using NMSDVDXLib;
using CDBurnerXP.Burning;

namespace CDBurnerXP.Controls
{
    internal class FileNode : IFileSystemNode
    {
        private string m_FileSystemPath = null;
        private DirectoryNode m_Parent = null;
        private FileInfo m_FileInfo = null;
        private bool m_IsImported = false;
        private string m_Name = null;
        private DateTime m_ImportedFileDate = DateTime.MinValue;
        private long m_ImportedFileSize = -1;
        private bool m_Hidden = false;

        #region Properties

        public bool Hidden
        {
            get
            {
                return m_Hidden;
            }
            set
            {
                m_Hidden = value;
            }
        }

        public long Size
        {
            set
            {
                m_ImportedFileSize = value;
            }
            get
            {
                if (m_FileInfo != null && m_FileInfo.Exists)
                {
                    return m_FileInfo.Length;
                }
                else if (m_ImportedFileSize > 0)
                {
                    return m_ImportedFileSize;
                }
                return 0L;
            }
        }

        public DateTime ModificationDate
        {
            set
            {
                m_ImportedFileDate = value;
            }
            get
            {
                if (m_ImportedFileDate != DateTime.MinValue)
                {
                    return m_ImportedFileDate;
                }
                else if (m_FileInfo != null)
                {
                    return m_FileInfo.LastWriteTime;
                }
                return DateTime.MinValue;
            }
        }

        public DirectoryNode Parent
        {
            get
            {
                return m_Parent;
            }
            set
            {
                m_Parent = value;
            }
        }

        public string Name
        {
            get
            {
                return m_Name;
            }
            set
            {
                if (m_Name != value)
                {
                    if (Parent != null)
                    {
                        string nameKey = m_Name.ToLower();
                        // Might not be present when loading a renamed file to the compilation
                        if (Parent.Files.ContainsKey(nameKey))
                        {
                            Parent.Files.Remove(nameKey);
                            Parent.Files.Add(value.ToLower(), this);
                        }
                    }
                    m_Name = value;
                }
            }
        }

        public bool IsImported
        {
            get
            {
                return m_IsImported;
            }
        }

        public string FileSystemPath
        {
            get
            {
                return m_FileSystemPath;
            }
            set
            {
                if (m_FileSystemPath != value)
                {
                    m_FileSystemPath = value;
                    if (m_FileInfo == null || m_FileInfo.FullName != m_FileSystemPath)
                    {
                        m_FileInfo = new FileInfo(m_FileSystemPath);
                    }
                }
            }
        }

        public string FullVirtualPath
        {
            get
            {
                string result = Path.Combine(m_Parent.FullVirtualPath, m_Name);
                return result.StartsWith("\\") ? result : "\\" + result;
            }
        }

        public FileInfo FileInfo
        {
            get
            {
                return m_FileInfo;
            }
        }

        #endregion

        public FileNode(string filePath, DirectoryNode dirNode, eVolumeTypeX volumeType)
        {
            m_FileInfo = new FileInfo(filePath);

            if (IsFileTooBig(m_FileInfo.Length, volumeType))
            {
                throw new FileTooBigException();
            }

            // Read test - we can't process files we may not read
            FileStream stream = File.OpenRead(filePath);
            stream.Close();

            m_Name = m_FileInfo.Name;
            m_FileSystemPath = m_FileInfo.FullName;
            m_Parent = dirNode;
        }

        public FileNode(string fileName, DirectoryNode dirNode, bool imported)
        {
            m_Name = fileName;
            m_FileSystemPath = "";
            m_Parent = dirNode;
            m_IsImported = imported;
        }

        /// <summary>
        /// Determines whether or not a file size is too big for
        /// a given file system type.
        /// </summary>
        private static bool IsFileTooBig(long lSize, NMSDVDXLib.eVolumeTypeX volumeType)
        {
            if (volumeType == NMSDVDXLib.eVolumeTypeX.vtUDF | volumeType == NMSDVDXLib.eVolumeTypeX.vtISO9660_Joliet_UDF | volumeType == NMSDVDXLib.eVolumeTypeX.vtISO9660_UDF)
            {
                // file is not too big when using UDF
                return false;
            }
            return (lSize > Convert.ToInt64(int.MaxValue) * 2);
        }

        /// <summary>
        /// Removes the file from the parent directory.
        /// </summary>
        public bool Remove()
        {
            if (m_Parent != null)
            {
                m_Parent.Files.Remove(Name.ToLower());
                return true;
            }
            else
            {
                return false;
            }
        }

        public override string ToString()
        {
            return FullVirtualPath;
        }
    }
}
