﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.ComponentModel;
using CDBurnerXP.IO;

namespace CDBurnerXP.Controls
{
    /// <summary>
    /// Base class for LayoutManagers
    /// </summary>
    public abstract class LayoutManager : Component
    {
        protected bool m_CancelAddingFiles = false;
        protected bool m_UnsavedChanges = false;
        protected string m_SavePath = null;
        protected bool m_SuppressChangedEvent = false;

        public event EventHandler<CompilationChangedEventArgs> CompilationChanged;

        #region CompilationChangedEventArgs

        public class CompilationChangedEventArgs : EventArgs
        {
            private bool m_ContentsChanged = false;

            /// <summary>
            /// Specifies whether or not the actual contents
            /// changed (so whether or not the size needs to be recalculated).
            /// </summary>
            public bool ContentsChanged
            {
                get
                {
                    return m_ContentsChanged;
                }
            }

            public CompilationChangedEventArgs(bool contentsChanged)
            {
                m_ContentsChanged = contentsChanged;
            }
        }

        #endregion

        #region Properties

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public abstract string LayoutName
        {
            get;
            set;
        }

        public abstract ISynchronizeInvoke SynchronizedObject
        {
            get;
        }

        public bool CancellationPending
        {
            get
            {
                return m_CancelAddingFiles;
            }
        }

        public string SavePath
        {
            get
            {
                return m_SavePath;
            }
        }

        public bool IsUnsaved
        {
            get
            {
                return m_UnsavedChanges;
            }
        }

        #endregion

        #region Event handling

        protected void OnCompilationChanged()
        {
            OnCompilationChanged(true, true);
        }

        protected void OnCompilationChanged(bool contentsChanged, bool setUnsaved)
        {
            if (m_SuppressChangedEvent) return;

            // Go to the thread of the form if necessary
            if (SynchronizedObject != null && SynchronizedObject.InvokeRequired)
            {
                SynchronizedObject.Invoke(new MethodInvoker(OnCompilationChanged), new object[] { contentsChanged });
                return;
            }

            m_UnsavedChanges = setUnsaved;

            if (CompilationChanged != null)
            {
                CompilationChanged(this, new CompilationChangedEventArgs(contentsChanged));
            }
        }

        #endregion

        /// <summary>
        /// If files are currently being added asynchronously,
        /// the operation will be cancelled.
        /// </summary>
        protected void CancelAddingFiles()
        {
            m_CancelAddingFiles = true;
            PathEx.CancelCopy = true;
        }

        /// <summary>
        /// Clears the whole layout.
        /// </summary>
        abstract public void Clear();

        /// <summary>
        /// Clears the whole layout.
        /// </summary>
        /// <param name="newLayoutName">A name to set for the new layout</param>
        abstract public void Clear(string newLayoutName);
    }
}
