﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using CDBurnerXP.Burning;
using NMSDVDXLib;
using System.Windows.Forms;
using RocketDivision.StarBurnX;
using System.Runtime.InteropServices;
using System.Threading;
using CDBurnerXP.Forms;

namespace CDBurnerXP.Controls
{
    /// <summary>
    /// Represents a combobox (+ additonal controls) which allow the user to
    /// select a speed for burning.
    /// </summary>
    public partial class SpeedSelect : System.Windows.Forms.UserControl
    {
        private string m_DeviceLetter;
        private string m_BurnSpeedLabel = "Write Speed";
        private static string m_MaxSpeedLabel = "Maximum";
        private bool m_BurnSpeedLabelEnabled = true;
        private BindingList<BurnSpeed> m_SpeedColl = new BindingList<BurnSpeed>();
        private string m_WriteSpeedDescription = "";
        private int m_WriteSpeedLongValue = 0;
        private int m_WriteSpeedShortValue = 0;
        private long m_SetLastWriteSpeed = 0;
        private double m_LastSelectedSpeed = 0;

        public event SpeedChangedEventHandler SpeedChanged;
        public delegate void SpeedChangedEventHandler(object sender, SpeedChangeEventArgs e);


        #region "Design time properties"

        /// <summary>
        /// Gets or sets the text of the burn speed combo box label
        /// </summary>
        [DefaultValue("Write Speed")]
        [Description("Gets or sets the text of the burn speed combo box label")]
        public string BurnSpeedLabel
        {
            get { return m_BurnSpeedLabel; }
            set
            {
                m_BurnSpeedLabel = value;
                lblBurnSpeed.Text = m_BurnSpeedLabel;
            }
        }

        /// <summary>
        /// Enable or disable the text of the burn speed combo box label
        /// </summary>
        [DefaultValue(true)]
        [Description("Enable or disable the text of the burn speed combo box label")]
        public bool BurnSpeedLabelEnabled
        {
            get { return m_BurnSpeedLabelEnabled; }
            set
            {
                m_BurnSpeedLabelEnabled = value;
                lblBurnSpeed.Enabled = m_BurnSpeedLabelEnabled;
            }
        }

        #endregion

        #region "Properties"

        /// <summary>
        /// Gets or sets the string which is represents the maximum speed
        /// (if no speed can be determined for example).
        /// </summary>
        [Browsable(false)]
        public static string MaximumSpeedLabel
        {
            get { return SpeedSelect.m_MaxSpeedLabel; }
            set { SpeedSelect.m_MaxSpeedLabel = value; }
        }

        /// <summary>
        /// Sets the speed which should be preselected.
        /// </summary>
        [Browsable(false)]
        [Description("Set the current write speed as the actual speed in KB/s")]
        public long SetLastWriteSpeed
        {
            set
            {
                m_SetLastWriteSpeed = value;
                LoadLastWriteSpeed();
            }
        }

        /// <summary>
        /// Gets the current write speed as the actual speed in KB/s
        /// </summary>
        [Browsable(false)]
        [Description("Get the current write speed as the actual speed in KB/s")]
        public int WriteSpeedLongValue
        {
            get { return m_WriteSpeedLongValue; }
        }

        /// <summary>
        /// Get the description for the current write speed
        /// </summary>
        [Browsable(false)]
        [Description("Get the description for the current write speed")]
        public string WriteSpeedDescription
        {
            get { return m_WriteSpeedDescription; }
        }

        /// <summary>
        /// Set the burning device for the speed selection.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Description("Set the burning device for the speed selection")]
        public string BurnSpeedDeviceLetter
        {
            set
            {
                if (value == null) return;


                if (value != m_DeviceLetter)
                {
                    m_DeviceLetter = value;
                    LoadBurnSpeedCombo(LoadSpeedCollection(true));
                    cboBurnSpeed_SelectedIndexChanged(this, null);
                }
            }
            get
            {
                return m_DeviceLetter;
            }
        }

        #endregion

        /// <summary>
        /// Sets the preselected write speed.
        /// </summary>
        /// <see cref="SetLastWriteSpeed"/>
        private void LoadLastWriteSpeed()
        {
            if (cboBurnSpeed.Items.Count > 0)
            {
                foreach (BurnSpeed bs in cboBurnSpeed.Items)
                {
                    if (bs.SpeedLongValue == m_SetLastWriteSpeed)
                    {
                        cboBurnSpeed.SelectedItem = bs;
                        return;
                    }
                }
            }
        }

        /// <summary>
        /// Loads the new speeds (after media change) in background.
        /// </summary>
        private void BeginLoadBurnSpeedCombo()
        {
            Thread thread = new Thread(new ThreadStart(delegate()
            {
                BindingList<BurnSpeed> speeds = LoadSpeedCollection(true);
                try
                {
                    this.Invoke((MethodInvoker)delegate()
                    {
                        LoadBurnSpeedCombo(speeds);
                    });
                }
                catch (ObjectDisposedException)
                {
                    // Ignore if thread finished too late
                }
                catch (InvalidOperationException)
                {
                    // Ignore if thread started too early
                }
            }));
            thread.IsBackground = true;
            thread.Start();
        }

        /// <summary>
        /// Loads the new speeds immediately (without refreshing disc information though).
        /// </summary>
        public void UpdateBurningSpeeds()
        {
            BindingList<BurnSpeed> speeds = LoadSpeedCollection(false);
            if (speeds.Count == 1 && speeds[0].SpeedLongValue == 0)
            {
                speeds = LoadSpeedCollection(true);
            }

            LoadBurnSpeedCombo(speeds);
        }

        /// <summary>
        /// Fills the ComboBox with the given speeds.
        /// </summary>
        /// <param name="speedColl">Speeds to allow for selection</param>
        private void LoadBurnSpeedCombo(BindingList<BurnSpeed> speedColl)
        {
            this.cboBurnSpeed.SelectedIndexChanged -= cboBurnSpeed_SelectedIndexChanged;
            try
            {
                bool wasNullSpeed = (WriteSpeedLongValue == 0);

                // Make sure that last custom speed is still available
                if (cboBurnSpeed.SelectedItem != null)
                {
                    BurnSpeed burnSpeed = (BurnSpeed)cboBurnSpeed.SelectedItem;
                    if (burnSpeed.IsCustom)
                    {
                        speedColl.Add(burnSpeed);
                    }
                }

                // Allow users to choose custom speeds if necessary.
                speedColl.Add(new BurnSpeed() { SpeedDescription = BaseForm.RM.GetString("<Other speed...>"), SpeedShortValue = -1, SpeedLongValue = -1 });

                // Set up ComboBox data binding
                cboBurnSpeed.DataSource = speedColl;
                cboBurnSpeed.ValueMember = "SpeedLongValue";
                cboBurnSpeed.DisplayMember = "SpeedDescription";

                if (cboBurnSpeed.Items.Count > 0)
                {
                    if (m_LastSelectedSpeed > 0)
                    {
                        foreach (BurnSpeed item in this.cboBurnSpeed.Items)
                        {
                            if (item.SpeedShortValue == m_LastSelectedSpeed)
                            {
                                cboBurnSpeed.SelectedItem = item;
                                break;
                            }
                        }
                    }

                    if (cboBurnSpeed.SelectedIndex == -1)
                    {
                        cboBurnSpeed.SelectedIndex = 0;
                    }

                    // Fire event
                    BurnSpeed newSpeed = (BurnSpeed)cboBurnSpeed.Items[0];
                    if (wasNullSpeed || m_LastSelectedSpeed != newSpeed.SpeedShortValue)
                    {
                        cboBurnSpeed_SelectedIndexChanged(this, null);
                    }
                }
            }
            finally
            {
                this.cboBurnSpeed.SelectedIndexChanged += cboBurnSpeed_SelectedIndexChanged;
            }
        }

        /// <summary>
        /// Returns all speeds which are currently supported by the drive.
        /// </summary>
        private BindingList<BurnSpeed> LoadSpeedCollection(bool refreshDisc)
        {
            List<BurnSpeed> speedColl = new List<BurnSpeed>();

            Drive drive = BurningEngine.GetStarBurnDriveByLetter(m_DeviceLetter);
            if (drive == null)
            {
                // Device not detected (maybe only present in NMS), use max speed.
                return GetNullBurnSpeed();
            }

            if (refreshDisc)
            {
                try
                {
                    drive.DriveInfo.Refresh();
                }
                catch (COMException)
                {
                    // Device not accessible
                    return GetNullBurnSpeed();
                }

                try
                {
                    // Refresh disc info for determining whether or not a disc
                    // has been inserted.
                    drive.DiscInfo.Refresh();
                }
                catch (COMException)
                {
                    // Not critical.
                }
            }

            if (drive.DriveInfo.SupportedWriteSpeeds.Count == 0)
            {
                // No reported writespeed
                return GetNullBurnSpeed();
            }

            foreach (DriveSpeed supportedSpeed in drive.DriveInfo.SupportedWriteSpeeds)
            {
                BurnSpeed speed = new BurnSpeed();
                speed.SpeedLongValue = supportedSpeed.Speed;
                speed.SpeedShortValue = supportedSpeed.SpeedRatio;
                speed.SpeedDescription = speed.SpeedShortValue.ToString() + "x (" + speed.SpeedLongValue.ToString("0,0") + " KB/s)";
                speed.IsValidated = (drive.DiscInfo.DiscType != (int)STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_NONE);
                speedColl.Add(speed);
            }

            speedColl.Sort();

            return new BindingList<BurnSpeed>(speedColl);
        }

        /// <summary>
        /// Returns a list with just a single write speed, which is
        /// "Maximum" (for cases where the burning speed is not known).
        /// </summary>
        private static BindingList<BurnSpeed> GetNullBurnSpeed()
        {
            BindingList<BurnSpeed> speedColl = new BindingList<BurnSpeed>();
            BurnSpeed speed = new BurnSpeed();
            speed.SpeedDescription = m_MaxSpeedLabel;
            speedColl.Add(speed);
            return speedColl;
        }

        private void MediaInserted(string label)
        {
            if (this.IsDisposed) return;

            if (BurningEngine.DeviceLettersEqual(label, m_DeviceLetter))
            {
                BeginLoadBurnSpeedCombo();
            }
        }

        private void MediaRemoved(string label)
        {
            if (this.IsDisposed) return;

            if (BurningEngine.DeviceLettersEqual(label, m_DeviceLetter))
            {
                // Disc removed: Only show "maximum"
                LoadBurnSpeedCombo(GetNullBurnSpeed());
            }
        }

        private void cboBurnSpeed_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            SpeedChangeEventArgs arg = new SpeedChangeEventArgs();
            BurnSpeed bs = default(BurnSpeed);
            if (cboBurnSpeed.Items.Count > 0)
            {
                bs = (BurnSpeed)cboBurnSpeed.SelectedItem;

                // Choose custom speed?
                if (bs.SpeedLongValue == -1 && this.cboBurnSpeed.Focused)
                {
                    using (CustomSpeedDialog dialog = new CustomSpeedDialog())
                    {
                        Drive drive = BurningEngine.GetStarBurnDriveByLetter(m_DeviceLetter);
                        if (drive != null)
                        {
                            dialog.DiscType = (STARBURN_DISC_TYPE)drive.DiscInfo.DiscType;
                        }

                        if (dialog.ShowDialog(this) == DialogResult.OK)
                        {
                            BindingList<BurnSpeed> speeds = cboBurnSpeed.DataSource as BindingList<BurnSpeed>;
                            BurnSpeed selectedSpeed = dialog.SelectedSpeed;

                            selectedSpeed.IsCustom = true;
                            selectedSpeed.SpeedDescription = BaseForm.RM.GetString("Other:") + " " + selectedSpeed.SpeedDescription;

                            // Search for existing custom speed and replace, otherwise insert new.
                            for (int i = 0; i < speeds.Count; i++)
                            {
                                if (speeds[i].IsCustom)
                                {
                                    speeds[i] = selectedSpeed;
                                    cboBurnSpeed.SelectedIndex = i;
                                    return;
                                }
                            }
                            
                            speeds.Insert(0, selectedSpeed);
                        }

                        cboBurnSpeed.SelectedIndex = 0;
                        return;
                    }
                }

                m_WriteSpeedDescription = bs.SpeedDescription;
                m_WriteSpeedLongValue = bs.SpeedLongValue;
                m_WriteSpeedShortValue = Convert.ToInt32(bs.SpeedShortValue);

                arg.speedDescription = m_WriteSpeedDescription;
                arg.speedLongValue = m_WriteSpeedLongValue;

                // Save selected speed for CD switch
                if (bs.SpeedShortValue > 0 && bs.IsValidated)
                {
                    m_LastSelectedSpeed = bs.SpeedShortValue;
                }
            }
            if (SpeedChanged != null)
            {
                SpeedChanged(sender, arg);
            }
        }

        public SpeedSelect()
        {
            // This call is required by the Windows Form Designer.
            InitializeComponent();
            MediaNotifier.AnyMediaInserted += MediaInserted;
            MediaNotifier.AnyMediaRemoved += MediaRemoved;
        }
    }

    /// <summary>
    /// Represents a burning speed with different units.
    /// </summary>
    public struct BurnSpeed : IComparable<BurnSpeed>
    {
        private string m_SpeedDescription;
        private int m_SpeedLongValue;
        private double m_SpeedShortValue;

        /// <summary>
        /// Gets or sets if the speed has been determined while a disc was actually inserted.
        /// </summary>
        public bool IsValidated { get; set; }

        public bool IsCustom { get; set; }

        public string SpeedDescription
        {
            get { return m_SpeedDescription; }
            set { m_SpeedDescription = value; }
        }

        public int SpeedLongValue
        {
            get { return m_SpeedLongValue; }
            set { m_SpeedLongValue = value; }
        }

        public double SpeedShortValue
        {
            get { return m_SpeedShortValue; }
            set { m_SpeedShortValue = value; }
        }

        public int CompareTo(BurnSpeed other)
        {
            return other.m_SpeedLongValue.CompareTo(this.m_SpeedLongValue);
        }
    }

    /// <summary>
    /// Contains information about the speed selection change.
    /// </summary>
    public class SpeedChangeEventArgs : EventArgs
    {
        public string speedDescription;
        public int speedLongValue;
    }
}