﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NMSDVDXLib;
using CDBurnerXP.Burning;
using CDBurnerXP.IO;
using StarBurnX = RocketDivision.StarBurnX;
using System.Runtime.InteropServices;

namespace CDBurnerXP.Forms
{
    public partial class CheckDisc : BaseForm
    {
        #region Enums

        public enum DiscType
        {
            Any = 0,
            CD,
            DVD
        }

        [Flags()]
        public enum DiscFeature
        {
            Any = 0,
            Writable = 1,
            Rewritable = 2,
            Empty = 4
        }

        #endregion

        private DiscFeature m_RequiredDiscFeature = DiscFeature.Any;
        private DiscType m_RequiredType = DiscType.Any;
        private object m_CurrentDevice = null;
        private long m_RequiredSpace = 0;
        private bool? m_LastStatus = null;
        private AudioBurner m_AudioImage = null;
        private bool m_DoClose = false;
        private bool m_AskDiscAtOnce = false;
        private IWin32Window m_OwnerWindow = null;
        private bool m_DisableCheckShowingDialog = false;
        private bool m_IsImported = false;
        private bool m_SuggestDiscSpanning = false;
        private bool m_DiscSpanningSelected = false;
        private bool m_DoNotAskEraseAgain = false;
        private bool m_AllowNextOkAsOverburning = false;
        public const int AUDIO_SEC_PER_TRACK = 75;

        #region Properties

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        public IWin32Window OwnerWindow
        {
            get { return m_OwnerWindow; }
            set { m_OwnerWindow = value; }
        }

        public AudioBurner AudioImage
        {
            get { return m_AudioImage; }
            set { m_AudioImage = value; }
        }

        /// <summary>
        /// True, if the user selected disc spanning from the menu.
        /// </summary>
        public bool DiscSpanningSelected
        {
            get { return m_DiscSpanningSelected; }
        }

        /// <summary>
        /// Gets or sets whether "Disc spanning" should be an option
        /// offered in the dropdown menu.
        /// </summary>
        public bool SuggestDiscSpanning
        {
            get { return m_SuggestDiscSpanning; }
            set { m_SuggestDiscSpanning = value; }
        }

        /// <summary>
        /// Gets or sets whether or not to ask for
        /// disc-at-once mode if the audio image is too huge
        /// for track-at-once.
        /// </summary>
        public bool AskDiscAtOnce
        {
            get { return m_AskDiscAtOnce; }
            set { m_AskDiscAtOnce = value; }
        }

        /// <summary>
        /// Gets or sets whether or not earlier sessions have been
        /// imported. In this case, we do not want to erase the disc.
        /// </summary>
        public bool IsImported
        {
            get { return m_IsImported; }
            set { m_IsImported = value; }
        }

        public long RequiredSpace
        {
            get
            {
                return m_RequiredSpace;
            }
            set
            {
                m_RequiredSpace = value;
            }
        }

        public object CurrentDevice
        {
            set
            {
                m_CurrentDevice = value;
            }
        }

        public DiscFeature RequiredDiscFeature
        {
            get
            {
                return m_RequiredDiscFeature;
            }
            set
            {
                m_RequiredDiscFeature = value;
            }
        }

        public DiscType RequiredType
        {
            get
            {
                return m_RequiredType;
            }
            set
            {
                m_RequiredType = value;
            }
        }

        #endregion

        #region Compatibility Methods

        /// <summary>
        /// Ejects the disc.
        /// </summary>
        private void Eject()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                drive.Eject();
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                device.Eject();
            }
        }

        /// <summary>
        /// Refreshes the current disc contents.
        /// </summary>
        private void RefreshDisc()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                drive.DiscInfo.Refresh();
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                device.Refresh(true);
            }
        }

        /// <summary>
        /// Returns a human readable name for the current disc type.
        /// </summary>
        /// <returns></returns>
        private string GetDiscDescription()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                if (drive.State != RocketDivision.StarBurnX.STARBURN_DRIVE_STATE.STARBURN_DRIVE_STATE_READY)
                {
                    //return BurningEngine.GetDiscDescription(StarBurnX.STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_UNKNOWN);
                }

                try
                {
                    return BurningEngine.GetDiscDescription((StarBurnX.STARBURN_DISC_TYPE)drive.DiscInfo.DiscType);
                }
                catch (COMException)
                {
                    // Error reading disc, etc.
                    return BurningEngine.GetDiscDescription(StarBurnX.STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_UNKNOWN);
                }
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                return BurningEngine.GetDiscDescription(device.GetDiscInfo().GetMediaType());
            }

            return null;
        }

        /// <summary>
        /// Returns whether or not a CD-type media is currently inserted.
        /// </summary>
        private bool IsCDMediaType()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                if (drive.State != RocketDivision.StarBurnX.STARBURN_DRIVE_STATE.STARBURN_DRIVE_STATE_READY)
                {
                    //return false;
                }

                return (drive.DiscInfo.DiscType & (int)StarBurnX.STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_CD) > 0;
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
               return device.GetDiscInfo().IsCDMediaType();
            }

            return false;
        }

        /// <summary>
        /// Returns whether or not a DVD-type media is currently inserted.
        /// </summary>
        private bool IsDvdMediaType()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                if (drive.State != RocketDivision.StarBurnX.STARBURN_DRIVE_STATE.STARBURN_DRIVE_STATE_READY)
                {
                    //return false;
                }

                return (drive.DiscInfo.DiscType & (int)StarBurnX.STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_DVD) > 0;
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                return device.GetDiscInfo().IsDVDMediaType();
            }

            return false;
        }

        /// <summary>
        /// Returns whether or not the currently inserted disc is still open / not finalized.
        /// </summary>
        private bool IsDiscOpen()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                if (drive.State != RocketDivision.StarBurnX.STARBURN_DRIVE_STATE.STARBURN_DRIVE_STATE_READY)
                {
                    //return false;
                }
                return (drive.DiscInfo.Status != RocketDivision.StarBurnX.STARBURN_DISC_STATUS.STARBURN_DISC_STATUS_COMPLETE && drive.DiscInfo.TotalSize > 0);
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                return device.GetDiscInfo().IsOpen();
            }

            return false;
        }

        /// <summary>
        /// Returns whether or not the currently inserted disc can be erased.
        /// </summary>
        private bool IsDiscErasable()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                if (drive.State != RocketDivision.StarBurnX.STARBURN_DRIVE_STATE.STARBURN_DRIVE_STATE_READY)
                {
                    //return false;
                }
                return (drive.DiscInfo.DiscType & (int)StarBurnX.STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_ERASABLE) > 0;
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                return device.GetDiscInfo().IsErasable();
            }

            return false;
        }

        /// <summary>
        /// Determines the currently free disc space.
        /// </summary>
        /// <returns>Size in bytes</returns>
        private long GetFreeDiscSpace()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                if (drive.State != RocketDivision.StarBurnX.STARBURN_DRIVE_STATE.STARBURN_DRIVE_STATE_READY)
                {
                    //return 0;
                }
                return (FormatFileSize.BlocksToDataBytes(drive.DiscInfo.TotalSize) - FormatFileSize.BlocksToDataBytes(drive.DiscInfo.UsedSize));
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                int availableBlocks = device.GetCapacityBlocks(true) - device.GetUsedSpaceBlocks(true);
                return availableBlocks * 2048L /* block size usually */;
            }

            return 0L;
        }

        /// <summary>
        /// Determines the currently used disc space.
        /// </summary>
        /// <returns>Size in bytes</returns>
        private long GetUsedDiscSpace()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                return FormatFileSize.BlocksToDataBytes(drive.DiscInfo.UsedSize);
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                return device.GetCapacityBlocks(true) * 2048L;
            }

            return 0L;
        }
       
        /// <summary>
        /// Determines the currently fee time on the disc.
        /// </summary>
        private TimeSpan GetFreeDiscTime()
        {
            StarBurnX.Drive drive = m_CurrentDevice as StarBurnX.Drive;
            if (drive != null)
            {
                if (drive.State != RocketDivision.StarBurnX.STARBURN_DRIVE_STATE.STARBURN_DRIVE_STATE_READY)
                {
                    //return TimeSpan.FromSeconds(0);
                }

                int seconds = (drive.DiscInfo.TotalSize - drive.DiscInfo.UsedSize) / CheckDisc.AUDIO_SEC_PER_TRACK;
                return TimeSpan.FromSeconds(seconds);
            }

            DeviceX device = m_CurrentDevice as DeviceX;
            if (device != null)
            {
                int availableBlocks = device.GetCapacityBlocks(true) - device.GetUsedSpaceBlocks(true);

                MSFX availableTime = new MSFX();
                availableTime.SetLBA(availableBlocks);
                return new TimeSpan(0, availableTime.GetMinutes(), availableTime.GetSeconds());
            }

            return TimeSpan.Zero;
        }

        /// <summary>
        /// Determines the drive letter of the current drive.
        /// </summary>
        private static string GetDriveLetter(object input)
        {
            StarBurnX.Drive drive = input as StarBurnX.Drive;
            if (drive != null)
            {
                return drive.DriveInfo.Letter;
            }

            DeviceX device = input as DeviceX;
            if (device != null)
            {
                return device.GetDriveLetter();
            }

            return null;
        }

        #endregion

        private CheckDisc()
        {
            InitializeComponent();
            CancelButton = bCancel;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            if (DesignMode) return;

            mnuOverburn.Text = RM.GetString("&Overburn");
            mnuRefresh.Text = RM.GetString("Re&fresh");
            mnuErase.Text = RM.GetString("Erase Disc...");
            mnuDiscSpanning.Text = RM.GetString("Disc spa&nning...");
            mnuDiscSpanning.Visible = m_SuggestDiscSpanning;

            MediaNotifier.AnyMediaInfoLoaded += new MediaNotifier.MediaInfoLoadedEventHandler(OnAnyMediaInfoLoaded);
            // Disc has already been checked before showing the dialog, do not check disc now.
            this.CheckRequirements(false);
            this.tmrManualRefresh.Start();
        }

        protected override void OnClosed(EventArgs e)
        {
            this.tmrManualRefresh.Stop();
            MediaNotifier.AnyMediaInfoLoaded -= new MediaNotifier.MediaInfoLoadedEventHandler(OnAnyMediaInfoLoaded);
            base.OnClosed(e);
        }

        private void OnAnyMediaInfoLoaded(StarBurnX.DiscInfo discInfo)
        {
            try
            {
                this.Invoke((MethodInvoker)delegate
                {
                    this.UpdateStatus(false);
                });
            }
            catch (InvalidOperationException)
            {
            }
        }

        private void UpdateStatus(bool refreshDiscInfo)
        {
            m_LastStatus = null;
            Cursor = Cursors.AppStarting;

            try
            {
                using (new ControlRedrawLock(this))
                {
                    if (CheckRequirements(refreshDiscInfo))
                    {
                        DialogResult = DialogResult.OK;
                        Close();
                        return;
                    }
                }
            }
            catch (COMException ex)
            {
                if (!HandleStarBurnError(this, ex, m_CurrentDevice))
                {
                    throw;
                }
            }
            finally
            {
                Cursor = Cursors.Default;
            }

            this.tmrManualRefresh.Start();
        }

        private void tmrManualRefresh_Tick(object sender, EventArgs e)
        {
            // Do not rely on auto-insert notification alone
            this.tmrManualRefresh.Stop();
            MediaNotifier.InvokeMediaInfoRefresh(Convert.ToChar(GetDriveLetter(m_CurrentDevice)));
        }

        private void bEject_Click(object sender, EventArgs e)
        {
            try
            {
                Eject();
            }
            catch (COMException ex)
            {
                if (!HandleStarBurnError(this, ex, m_CurrentDevice))
                {
                    throw;
                }
            }
        }

        private void mnuOverburn_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
            return;
        }

        private void mnuRefresh_Click(object sender, EventArgs e)
        {
            UpdateStatus(true);
        }

        private void bActions_Click(object sender, EventArgs e)
        {
            if (bActions.Text == mnuDiscSpanning.Text)
            {
                mnuDiscSpanning.PerformClick();
            }
            else
            {
                UpdateStatus(true);
            }
        }

        private void mnuErase_Click(object sender, EventArgs e)
        {
            using (EraseDiscDialog dialog = new EraseDiscDialog(GetDriveLetter(m_CurrentDevice), null, false))
            {
                if (dialog.ShowDialog(this) == DialogResult.OK)
                {
                    UpdateStatus(true);
                }
            }
        }

        private void mnuDiscSpanning_Click(object sender, EventArgs e)
        {
            this.Close();
            m_DiscSpanningSelected = true;
        }

        /// <summary>
        /// Removes all status information from the table layout panel.
        /// </summary>
        private void ClearTable()
        {
            List<Control> remove = new List<Control>();

            foreach (Control c in tblRequirements.Controls)
            {
                if (c != lblRequired && c != lblPresent)
                {
                    remove.Add(c);
                }
            }

            foreach (Control c in remove)
            {
                tblRequirements.Controls.Remove(c);
            }
        }

        private void AddRow(string required, string present, bool isOK)
        {
            Label req = new Label();
            req.Anchor = (AnchorStyles.Left);
            req.Padding = lblRequired.Padding;
            req.AutoSize = true;
            req.Text = required;

            Label pres = new Label();
            pres.AutoSize = true;
            pres.Padding = lblRequired.Padding;
            pres.Anchor = (AnchorStyles.Left);
            pres.Text = present;
            pres.ForeColor = (isOK) ? Color.Green : Color.Red;

            tblRequirements.RowStyles.Add(new RowStyle(SizeType.AutoSize));
            tblRequirements.Controls.Add(req, 0, -1);
            tblRequirements.Controls.Add(pres, 1, -1);
        }

        /// <summary>
        /// Checks the disc status, updates the UI.
        /// </summary>
        /// <returns>true, if all requirements are met, false otherwise</returns>
        private bool CheckRequirements(bool refreshDiscInfo)
        {
            if (m_DoClose)
            {
                DialogResult = DialogResult.Cancel;
                Close();
            }

            if (m_DisableCheckShowingDialog) return false;

            if (m_LastStatus.HasValue) return m_LastStatus.Value;

            bool result = true;
            if (refreshDiscInfo)
            {
                this.RefreshDisc();
            }

            string discDescription = GetDiscDescription();

            ClearTable();

            if (RequiredType != DiscType.Any)
            {
                if (RequiredType == DiscType.CD)
                {
                    bool condition = IsCDMediaType();
                    result = result && condition;
                    AddRow(RM.GetString("Type CD"), discDescription, condition);
                }
                if (RequiredType == DiscType.DVD)
                {
                    bool condition = IsDvdMediaType();
                    result = result && condition;
                    AddRow(RM.GetString("Type DVD"), discDescription, condition);
                }
            }

            if (RequiredDiscFeature != DiscFeature.Any)
            {
                if ((RequiredDiscFeature & DiscFeature.Writable) > 0)
                {
                    bool condition = IsDiscOpen();
                    string presentDesc = string.Format(condition ? RM.GetString("Writable ({0})") : RM.GetString("Not writable ({0})"), discDescription);
                    AddRow(RM.GetString("Writable disc"), presentDesc, condition);
                    // There still is a chance...but only if at least the medium type is right
                    if (!condition && IsDiscErasable() && result && !m_DisableCheckShowingDialog)
                    {
                        m_DisableCheckShowingDialog = true;

                        if (!m_DoNotAskEraseAgain && MessageBox.Show(OwnerWindow, RM.GetString("The media is not writable. Do you want to erase it now?"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                        {
                            try
                            {
                                using (EraseDiscDialog dialog = new EraseDiscDialog(GetDriveLetter(m_CurrentDevice), null, false))
                                {
                                    if (dialog.ShowDialog(OwnerWindow) == DialogResult.Cancel)
                                    {
                                        m_DoNotAskEraseAgain = true;
                                    }
                                }
                            }
                            finally
                            {
                                m_DisableCheckShowingDialog = false;
                            }
                            return CheckRequirements(true);
                        }
                        m_DisableCheckShowingDialog = false;
                    }
                    result = result && condition;
                }

                if ((RequiredDiscFeature & DiscFeature.Rewritable) > 0)
                {
                    bool condition = IsDiscErasable();
                    result = result && condition;
                    string presentDesc = string.Format(condition ? RM.GetString("Re-writable ({0})") : RM.GetString("Not re-writable ({0})"), discDescription);
                    AddRow(RM.GetString("Re-writable disc"), presentDesc, condition);
                }
            }

            // Only if the disc type is fine, we can show the Overburn button
            // Note: Overburning is only supported for DAO
            mnuOverburn.Enabled = result && (m_AudioImage == null || m_AudioImage.Mode == StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_96 || m_AudioImage.Mode == StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_DAO_16);

            if (RequiredSpace > 0 || m_AudioImage != null)
            {
                if (m_AudioImage != null)
                {
                    // Occasionally incorrect detection (http://forum.cdburnerxp.se/viewtopic.php?f=3&t=8741).
                    // We verify the disc size on our own, discard the burning library's opinion.
                    m_AudioImage.CheckOverburning = false;
   
                    TimeSpan availableTimeSpan = GetFreeDiscTime();

                    int requiredTime = 0;
                    foreach (RocketDivision.StarBurnX.AudioTrack track in m_AudioImage.Tracks)
                    {
                        requiredTime += track.Duration;
                    }
                    // Add pauses between tracks
                    if (m_AudioImage.Mode == StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO) 
                    {
                        requiredTime += (m_AudioImage.Tracks.Count - 1) * m_AudioImage.AudioPauseLength;
                    }

                    TimeSpan requiredTimeSpan = new TimeSpan(0, 0, requiredTime);

                    bool condition = (availableTimeSpan > requiredTimeSpan);
                    result = result && condition;

                    string req = string.Format(RM.GetString("{0:00}:{1:00} free"), Math.Floor(requiredTimeSpan.TotalMinutes), requiredTimeSpan.Seconds);
                    AddRow(req, string.Format(RM.GetString("{0:00}:{1:00}"), Math.Floor(availableTimeSpan.TotalMinutes), availableTimeSpan.Seconds), condition);
                }
                else
                {
                    long availableBytes = GetFreeDiscSpace();

                    bool condition = (availableBytes >= RequiredSpace);

                    // Suggest disc spanning if not enough free space and disc size insufficient
                    if (!condition && m_SuggestDiscSpanning && RequiredSpace > (GetFreeDiscSpace() + GetUsedDiscSpace()))
                    {
                        bActions.Text = mnuDiscSpanning.Text;
                    }
                    else
                    {
                        bActions.Text = mnuRefresh.Text;
                    }

                    result = result && condition;

                    string requiredFormatted;
                    string availableFormatted;
                    FormatFileSize.FormatForComparison(RequiredSpace, availableBytes, out requiredFormatted, out availableFormatted);
                    
                    string req = string.Format(RM.GetString("{0} free"), requiredFormatted);
                    AddRow(req, availableFormatted, condition);

                    long totalBytesOnDisc = GetUsedDiscSpace();
                    // Can we free up enough disc space when erasing the disc?
                    if (!condition && !IsImported && IsDiscErasable() && totalBytesOnDisc >= RequiredSpace)
                    {
                        m_DisableCheckShowingDialog = true;

                        if (!m_DoNotAskEraseAgain && MessageBox.Show(OwnerWindow, RM.GetString("There is not enough free space on the disc. Do you want to erase it now?"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                        {
                            try
                            {
                                using (EraseDiscDialog dialog = new EraseDiscDialog(GetDriveLetter(m_CurrentDevice), null, false))
                                {
                                    if (dialog.ShowDialog(OwnerWindow) == DialogResult.Cancel)
                                    {
                                        m_DoNotAskEraseAgain = true;
                                    }
                                }
                            }
                            finally
                            {
                                m_DisableCheckShowingDialog = false;
                            }
                            return CheckRequirements(true);
                        }
                        m_DisableCheckShowingDialog = false;
                    }
                }
            }

            // For audio CDs, additionally check that the disc
            // is completely empty (if one is present at all)
            // when using Disc at Once.
            if ((RequiredDiscFeature & DiscFeature.Empty) > 0 || (m_AudioImage != null && m_AudioImage.Mode != RocketDivision.StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO && IsCDMediaType()))
            {
                bool discIsEmpty = (GetUsedDiscSpace() == 0);
                result = result && discIsEmpty;
                if (!discIsEmpty) mnuOverburn.Enabled = false;
                AddRow(RM.GetString("Empty Disc"), discIsEmpty ? RM.GetString("Empty Disc") : RM.GetString("Not empty"), discIsEmpty);
            }

            m_LastStatus = result;
            return result;
        }

        /// <summary>
        /// Shows the disc check dialog for audio images.
        /// </summary>
        /// <param name="askDiscAtOnce">Whether or not to ask to continue if only DAO is possible</param>
        /// <returns>false if the user chose "cancel", true otherwise</returns>
        public static bool Show(IWin32Window owner, StarBurnX.Drive currentDevice, DiscFeature requiredFeature, AudioBurner audioImage, bool askDiscAtOnce, bool nextOkAsOverburning)
        {
            bool discSpanningSelected;
            return Show(owner, currentDevice, requiredFeature, DiscType.CD, 0, audioImage, askDiscAtOnce, false, nextOkAsOverburning, false, out discSpanningSelected);
        }

        /// <summary>
        /// Shows the disc check dialog for data images.
        /// </summary>
        /// <param name="requiredSpace">Required free space on disc in bytes</param>
        /// <returns>false if the user chose "cancel", true otherwise</returns>
        public static bool Show(IWin32Window owner, IDeviceX currentDevice, DiscFeature requiredFeature, DiscType requiredType, long requiredSpace, bool isImported, bool nextOkAsOverburning)
        {
            bool discSpanningSelected;
            return Show(owner, currentDevice, requiredFeature, requiredType, requiredSpace, null, false, isImported, nextOkAsOverburning, false, out discSpanningSelected);
        }

        /// <summary>
        /// Shows the disc check dialog for data images. Also suggests disc spanning if appropriate.
        /// </summary>
        /// <param name="requiredSpace">Required free space on disc in bytes</param>
        /// <returns>false if the user chose "cancel", true otherwise</returns>
        public static bool ShowWithDiscSpanning(IWin32Window owner, IDeviceX currentDevice, DiscFeature requiredFeature, DiscType requiredType, long requiredSpace, bool isImported, bool nextOkAsOverburning, out bool discSpanningSelected)
        {
            return Show(owner, currentDevice, requiredFeature, requiredType, requiredSpace, null, false, isImported, nextOkAsOverburning, true, out discSpanningSelected);
        }

        /// <summary>
        /// Shows the disc check dialog for data images.
        /// </summary>
        /// <param name="requiredSpace">Required free space on disc in bytes</param>
        /// <returns>false if the user chose "cancel", true otherwise</returns>
        public static bool Show(IWin32Window owner, StarBurnX.Drive currentDevice, DiscFeature requiredFeature, DiscType requiredType, long requiredSpace, bool isImported, bool nextOkAsOverburning)
        {
            bool discSpanningSelected;
            return Show(owner, currentDevice, requiredFeature, requiredType, requiredSpace, null, false, isImported, nextOkAsOverburning, false, out discSpanningSelected);
        }

        private static bool Show(IWin32Window owner, object currentDevice, DiscFeature requiredFeature, DiscType requiredType, long requiredSpace, AudioBurner audioImage, bool askDiscAtOnce, bool isImported, bool nextOkAsOverburning, bool suggestDiscSpanning, out bool discSpanningSelected)
        {
            discSpanningSelected = false;

            if (currentDevice == null)
            {
                ShowNoDeviceErrorMessage(owner);
                return false;
            }

            CheckDisc dialog = new CheckDisc();
            dialog.RequiredDiscFeature = requiredFeature;
            dialog.RequiredType = requiredType;
            dialog.CurrentDevice = currentDevice;
            dialog.RequiredSpace = requiredSpace;
            dialog.AudioImage = audioImage;
            dialog.OwnerWindow = owner;
            dialog.IsImported = isImported;
            dialog.SuggestDiscSpanning = suggestDiscSpanning;
            dialog.m_AllowNextOkAsOverburning = nextOkAsOverburning;

            bool notifierEnabled = MediaNotifier.Enabled;

            try
            {
                if (dialog.CheckRequirements(true))
                {
                    // Don't show dialog if everything is fine from the start
                    return true;
                }

                // In any case, we need the notifier now (for checking the disc periodically)
                MediaNotifier.DriveLetter = GetDriveLetter(currentDevice);
                MediaNotifier.Enabled = true;

                try
                {
                    return (dialog.ShowDialog(owner) == DialogResult.OK);
                }
                finally
                {
                    discSpanningSelected = dialog.DiscSpanningSelected;
                }
            }
            catch (COMException ex)
            {
                if (HandleStarBurnError(owner, ex, currentDevice))
                {
                    return false;
                }

                throw;
            }
            finally
            {
                dialog.Dispose();
                // Stop if previously disabled
                MediaNotifier.Enabled = notifierEnabled;
            }
        }

        /// <summary>
        /// Displays a more or less helpful error message when no devices are found.
        /// </summary>
        public static void ShowNoDeviceErrorMessage(IWin32Window owner)
        {
            // More helpful error message if no drives were found
            if (BurningEngine.GetBurningDeviceCount() == 0)
            {
                MessageBox.Show(owner, RM.GetString("Cannot continue with the burning process, because no burning device was detected.") + Environment.NewLine + Environment.NewLine + RM.GetString("Install a burning device and restart CDBurnerXP to burn a disc."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                MessageBox.Show(owner, RM.GetString("Cannot continue with the burning process, because no burning device was selected."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}