﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;
using CDBurnerXP.Burning;
using CDBurnerXP.IO;
using RocketDivision.StarBurnX;

namespace CDBurnerXP.Forms
{
    public partial class DiscInfoDialog : BaseForm
    {
        private string driveLetter;
        private Drive drive;

        #region Properties

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// Sets the drive letter, for the drive to check.
        /// </summary>
        public string DriveLetter
        {
            set
            {
                this.driveLetter = value;
            }
        }

        #endregion

        /// <summary>
        /// Represents a dialog which shows disc information.
        /// </summary>
        public DiscInfoDialog()
        {
            InitializeComponent();

            CancelButton = bClose;
            tvInfo.ContextMenu = cmuCopy;
            mnuCopy.Text = RM.GetString("Copy");
            MediaNotifier.AnyMediaInfoLoaded += new MediaNotifier.MediaInfoLoadedEventHandler(MediaNotifier_AnyMediaInfoLoaded);

            imlIcons.ColorDepth = ColorDepth.Depth32Bit;
            imlIcons.Images.Add("Info", CsLib.Properties.Resources.SymbolInformation);
            imlIcons.Images.Add("Yes", CsLib.Properties.Resources.SymbolCheck);
            imlIcons.Images.Add("No", CsLib.Properties.Resources.SymbolDelete);
            imlIcons.Images.Add("Session", CsLib.Properties.Resources.Session);
            imlIcons.Images.Add("Track", CsLib.Properties.Resources.Track);
        }

        /// <summary>
        /// Whenever a new disc is inserted, the dialog reads the new information.
        /// </summary>
        private void MediaNotifier_AnyMediaInfoLoaded(DiscInfo discInfo)
        {
            this.Invoke((MethodInvoker)delegate() {
                try
                {
                    RefreshDiscInfo(discInfo);
                }
                catch (COMException ex)
                {
                    HandleStarBurnError(this, ex, this.drive);
                }
            });
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            // Initially load drive information
            try
            {
                this.drive = BurningEngine.GetStarBurnDriveByLetter(this.driveLetter);
            }
            catch (COMException)
            {
                // Ignore, display "unknown"
            }

            if (drive == null)
            {
                lblDevice.Text = "(" + RM.GetString("Unknown") + ")";
                lblSize.Text = "(" + RM.GetString("Unknown") + ")";
                bRefreshDisc.Enabled = false;
                return;
            }

            // Drive label
            lblDevice.Text = BurningEngine.FormatDevice(this.drive);

            try
            {
                RefreshDiscInfo(this.drive.DiscInfo);
            }
            catch (COMException ex)
            {
                HandleStarBurnError(this, ex, this.drive);
            }
        }

        private void mnuCopy_Click(object sender, EventArgs e)
        {
            if (tvInfo.SelectedNode != null)
            {
                SafeClipboard.SetData(tvInfo.SelectedNode.Text, true);
            }
        }

        private void bRefreshDisc_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            bRefreshDisc.Enabled = false;

            // Refresh information in background
            Thread thread = new Thread(delegate()
            {
                try
                {
                    try
                    {
                        this.drive.DiscInfo.Refresh();
                        this.Invoke((MethodInvoker)delegate()
                        {
                            RefreshDiscInfo(this.drive.DiscInfo);
                        });
                    }
                    catch (COMException ex)
                    {
                        this.Invoke((MethodInvoker)delegate()
                        {
                            HandleStarBurnError(this, ex, this.drive);
                        });
                    }
                    finally
                    {
                        this.Invoke((MethodInvoker)delegate()
                        {
                            bRefreshDisc.Enabled = true;
                            Cursor = Cursors.Default;
                        });
                    }
                }
                catch (ObjectDisposedException)
                {
                }
                catch (InvalidOperationException)
                {
                }
            });
            thread.Start();
        }

        protected override void OnClosed(EventArgs e)
        {
            MediaNotifier.AnyMediaInfoLoaded -= MediaNotifier_AnyMediaInfoLoaded;

            base.OnClosed(e);
        }

        /// <summary>
        /// Fills the TreeView with information about the disc.
        /// </summary>
        private void RefreshDiscInfo(DiscInfo discInfo)
        {
            // Space bar info
            long totalBytes = FormatFileSize.BlocksToDataBytes(discInfo.TotalSize);
            long usedBytes = FormatFileSize.BlocksToDataBytes(discInfo.UsedSize);

            discSpaceBar.MaxValue = totalBytes;
            discSpaceBar.CurrentValue = usedBytes;

            lblSize.Text = string.Format(RM.GetString("{0} free of {1}"), FormatFileSize.Format(totalBytes - usedBytes), FormatFileSize.Format(totalBytes));


            tvInfo.BeginUpdate();
            try
            {
                tvInfo.Nodes.Clear();

                if (discInfo.DiscType == (int)STARBURN_DISC_STATUS.STARBURN_DISC_STATUS_EMPTY)
                {
                    tvInfo.Nodes.Add(RM.GetString("No Disc"));
                    return;
                }

                tvInfo.Nodes.Add(RM.GetString("Title:") + " " + (string.IsNullOrEmpty(discInfo.DiscLabel) ? "(" + RM.GetString("None") + ")" : discInfo.DiscLabel));
                tvInfo.Nodes.Add(RM.GetString("Total Space:") + " " + FormatFileSize.Format(totalBytes));
                tvInfo.Nodes.Add(RM.GetString("Used Space:") + " " + FormatFileSize.Format(usedBytes));
                tvInfo.Nodes.Add(RM.GetString("Free Space:") + " " + FormatFileSize.Format(totalBytes - usedBytes));
                tvInfo.Nodes.Add(RM.GetString("Type:") + " " + BurningEngine.GetDiscDescription((STARBURN_DISC_TYPE)discInfo.DiscType));

                // File system
                string fsText = RM.GetString("File system:") + " ";
                switch (discInfo.FileSystem)
                {
                    case STARBURN_DISC_FILESYSTEM.STARBURN_FILESYSTEM_ISO9660: fsText += "ISO9660"; break;
                    case STARBURN_DISC_FILESYSTEM.STARBURN_FILESYSTEM_ISO9660_JOLIET: fsText += "ISO9660 / Joliet"; break;
                    case STARBURN_DISC_FILESYSTEM.STARBURN_FILESYSTEM_UDF: fsText += "UDF"; break;
                    case STARBURN_DISC_FILESYSTEM.STARBURN_FILESYSTEM_UNKNOWN: fsText += RM.GetString("Unknown"); break;
                }
                tvInfo.Nodes.Add(fsText);

                // Bootable
                if (discInfo.IsDiscBootable)
                {
                    tvInfo.Nodes.Add("", RM.GetString("Bootable:") + " " + RM.GetString("Yes"), "Yes", "Yes");
                }
                else
                {
                    tvInfo.Nodes.Add("", RM.GetString("Bootable:") + " " + RM.GetString("No"), "No", "No");
                }

                // Disc status
                if (discInfo.Status == STARBURN_DISC_STATUS.STARBURN_DISC_STATUS_COMPLETE)
                {
                    tvInfo.Nodes.Add("", RM.GetString("Disc Is Open:") + " " + RM.GetString("No"), "No", "No");
                }
                else
                {
                    tvInfo.Nodes.Add("", RM.GetString("Disc Is Open:") + " " + RM.GetString("Yes"), "Yes", "Yes");
                }

                // Protection
                string protectionText = RM.GetString("Protection:") + " ";
                switch (discInfo.ProtectionSystem)
                {
                    case STARBURN_DISC_PROTECTION_SYSTEM.STARBURN_DVD_PROTECTION_CPRM: protectionText += "CPRM (Content Protection for Recordable Media)"; break;
                    case STARBURN_DISC_PROTECTION_SYSTEM.STARBURN_DVD_PROTECTION_CSS: protectionText += "CSS (Content Scramble System)"; break;
                    case STARBURN_DISC_PROTECTION_SYSTEM.STARBURN_DVD_PROTECTION_NONE: protectionText += RM.GetString("None"); break;
                }
                tvInfo.Nodes.Add(protectionText);

                // DVD regions
                if (discInfo.DVDRegionMask == 0)
                {
                    tvInfo.Nodes.Add(RM.GetString("DVD region: No information"));
                }
                else
                {
                    TreeNode node = tvInfo.Nodes.Add("DVD regions");
                    Dictionary<string, bool> regions = BurningEngine.GetRegionInfo(discInfo.DVDRegionMask);

                    foreach (KeyValuePair<string, bool> region in regions)
                    {
                        node.Nodes.Add(region.Key, RM.GetString(region.Key), region.Value ? "Yes" : "No", region.Value ? "Yes" : "No");
                    }

                    node.Expand();
                }

                // Sessions
                for (int i = 0; i < discInfo.Sessions.Count; i++)
                {
                    Session session = discInfo.Sessions[i];

                    TreeNode sessionNode = tvInfo.Nodes.Add("", RM.GetString("Session:") + String.Format(" {0:00}", i + 1), "Session", "Session");

                    for (int j = 0; j < session.Tracks.Count; j++)
                    {
                        Track track = session.Tracks[j];

                        TreeNode trackNode = sessionNode.Nodes.Add("", RM.GetString("Track:") + String.Format(" {0:00}", j + 1), "Track", "Track");

                        if (track.Audio)
                        {
                            trackNode.Nodes.Add(RM.GetString("Track Type:") + " " + RM.GetString("Audio"));
                        }
                        else
                        {
                            trackNode.Nodes.Add(RM.GetString("Track Type:") + " " + RM.GetString("Data"));
                        }

                        long size = ((long)track.LogicalBlockSize) * track.Size;
                        trackNode.Nodes.Add(RM.GetString("Size:") + " " + FormatFileSize.Format(size));

                        string modeText = RM.GetString("Mode:") + " ";
                        switch (track.TrackMode)
                        {
                            case STARBURN_TRACK_MODE.STARBURN_TRACK_MODE1: modeText += "Mode 1"; break;
                            case STARBURN_TRACK_MODE.STARBURN_TRACK_MODE2: modeText += "Mode 2"; break;
                            case STARBURN_TRACK_MODE.STARBURN_TRACK_UNDEFINED: modeText += RM.GetString("Unknown"); break;
                        }
                    }

                    sessionNode.Expand();
                }
            }
            finally
            {
                tvInfo.EndUpdate();
            }
        }
    }
}
