﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RocketDivision.StarBurnX;
using System.Threading;
using System.Runtime.InteropServices;
using CDBurnerXP.Controls;
using CDBurnerXP.Windows.Forms;
using CDBurnerXP.Burning;

namespace CDBurnerXP.Forms
{
    /// <summary>
    /// Represents a dialog which allows erasing discs.
    /// </summary>
    public partial class EraseDiscDialog : BaseForm
    {
        private DiscEraser discEreaser = new DiscEraser();
        private bool isSessionImported = false;
        private FileLayoutManager layoutManager = null;
        private bool isErasing = false;
        private bool isCancelled = false;
        private bool autoImportValue = false;

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        public EraseDiscDialog()
        {
            InitializeComponent();

            this.discEreaser.OnProgress += new IDiscEraserEvents_OnProgressEventHandler(discEreaser_OnProgress);
            AcceptButton = bErase;
            CancelButton = bCancel;
        }

        public EraseDiscDialog(string deviceLetter, FileLayoutManager layoutManager, bool isSessionImported)
            : this()
        {
            this.isSessionImported = isSessionImported;
            this.layoutManager = layoutManager;
            this.cboDrive.SelectedDeviceLetter = deviceLetter;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.autoImportValue = m_AutoImportEnabled;
            m_AutoImportEnabled = false;

            rbFast.Checked = Conversion.ToBoolean(Settings.GetValue("EraseMethodFastChecked", true));
            rbSecure.Checked = !Conversion.ToBoolean(Settings.GetValue("EraseMethodFastChecked", true));
            chkEjectDisc.Checked = Conversion.ToBoolean(Settings.GetValue("EjectAfterErase", false));
            
            // Only enable if drives are present
            bErase.Enabled = cboDrive.Enabled;
        }

        protected override void OnClosed(EventArgs e)
        {
            Settings.SetValue("EraseMethodFastChecked", rbFast.Checked);
            Settings.SetValue("EjectAfterErase", chkEjectDisc.Checked);

            base.OnClosed(e);

            this.discEreaser.OnProgress -= discEreaser_OnProgress;
            m_AutoImportEnabled = this.autoImportValue;
        }

        private void bErase_Click(object sender, EventArgs e)
        {
            // Safety prompt
            if (TaskDialog.ShowOptionalMessageBox(this, "BeforeErasing", RM.GetString("Erase Disc"), "", RM.GetString("Do you really want to erase the disc?"), eTaskDialogButtons.YesNo, eSysIcons.Warning, DialogResult.Yes) != DialogResult.Yes)
            {
                return;
            }

            if (this.isSessionImported)
            {
                string msgboxstr = RM.GetString("The layout contains an imported session from the media about to be erased.");
                msgboxstr += Environment.NewLine + RM.GetString("Erasing the media will clear the current layout. Do you want to continue?");
                if (MessageBox.Show(this, msgboxstr, RM.GetString("Clear Data Layout"), MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.No)
                {
                    return;
                }
                else
                {
                    this.isSessionImported = false;
                    layoutManager.Clear();
                }
            }

            if (!CheckDisc.Show(this, cboDrive.SelectedStarBurnDevice, CheckDisc.DiscFeature.Rewritable, CheckDisc.DiscType.Any, 0, false, false))
            {
                return;
            }

            ActivateControls(false);
            bCancel.Text = RM.GetString("Cancel");

            discEreaser.Drive = cboDrive.SelectedStarBurnDevice;

            Thread eraseThread = new Thread(new ParameterizedThreadStart(StartEraseThread));
            eraseThread.Start(rbSecure.Checked);
        }

        private void ActivateControls(bool enable)
        {
            bErase.Enabled = enable;
            cboDrive.Enabled = enable;
            rbFast.Enabled = enable;
            rbSecure.Enabled = enable;
        }

        private void StartEraseThread(object fullErase)
        {
            try
            {
                this.isCancelled = false;
                this.isErasing = true;
                BurningEngine.TryLock(this.discEreaser.Drive);
                this.discEreaser.Erase((bool)fullErase);
                HandleEraseComplete();
            }
            catch (COMException ex)
            {
                this.Invoke((MethodInvoker)delegate()
                {
                    string msg = RM.GetString("Erasing the disc failed for the following reason:") + " " + ex.Message;
                    MessageBox.Show(this, msg, RM.GetString("Erase Failed"), MessageBoxButtons.OK, MessageBoxIcon.Error);
                });
            }
            finally
            {
                BurningEngine.TryUnlock(this.discEreaser.Drive);
                this.isErasing = false;
            }
        }

        private void HandleEraseComplete()
        {
            if (InvokeRequired)
            {
                this.Invoke((MethodInvoker)HandleEraseComplete);
                return;
            }

            this.isErasing = false;
            if (chkEjectDisc.Checked && !this.isCancelled)
            {
                try
                {
                    this.discEreaser.Drive.Eject();
                }
                catch (COMException ex)
                {
                    HandleStarBurnError(this, ex, this.discEreaser.Drive);
                }
            }

            string msg = this.isCancelled ? RM.GetString("The erasing process has been cancelled.") : RM.GetString("The disc has been erased successfully.");
            TaskDialog.ShowOptionalMessageBox(this, "AfterErase", RM.GetString("Erase Finished"), "", msg, eTaskDialogButtons.OK, eSysIcons.Information, 0);
            DialogResult = this.isCancelled ? DialogResult.Cancel : DialogResult.OK;
        }

        private void discEreaser_OnProgress(int Percent, int TimeRemaining)
        {
            this.Invoke((MethodInvoker)delegate()
            {
                prgAction.Value = Percent;
            });
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);

            if (this.isErasing)
            {
                e.Cancel = true;

                if (!this.isCancelled)
                {
                    if (MessageBox.Show(this, RM.GetString("Do you really want to stop erasing the disc?"), RM.GetString("Cancel"), MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    {
                        this.isCancelled = true;
                        this.bCancel.Enabled = false;
                        DialogResult = DialogResult.None;
                        new Thread(new ThreadStart(this.discEreaser.Cancel)).Start();
                    }
                }
            }
        }
    }
}
