﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CDBurnerXP.Controls;
using CDBurnerXP.IO;
using Microsoft.Win32;
using NMSDVDXLib;

namespace CDBurnerXP.Forms
{
    public partial class PropertyDialog : BaseForm
    {
        private IFileSystemNode[] m_FileNodes;

        #region Properties

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        internal IFileSystemNode[] FileNodes
        {
            get
            {
                return m_FileNodes;
            }
            set
            {
                if (m_FileNodes == value) return;

                m_FileNodes = value;

                if (m_FileNodes.Length > 1)
                {
                    txtName.Visible = false;

                    int folderCount = 0;
                    int fileCount = 0;
                    foreach (IFileSystemNode node in m_FileNodes)
                    {
                        if (node is FileNode) fileCount++;
                        if (node is DirectoryNode) folderCount++;
                    }
                    lblNumFilesFolders.Text = FormatTranslation("{0} file(s) and {1} folder(s)", fileCount, folderCount);

                    long size = 0;
                    // Calculate total size
                    foreach (IFileSystemNode node in m_FileNodes)
                    {
                        size += node.Size;
                    }
                    lblSizeValue.Text = FormatFileSize.Format(size);

                    // Check whether or not all dates are equal
                    bool allEqual = true;
                    DateTime firstVal = m_FileNodes[0].ModificationDate;
                    foreach (IFileSystemNode node in m_FileNodes)
                    {
                        if (node.ModificationDate != firstVal)
                        {
                            allEqual = false;
                        }
                    }
                    dtpDate.Value = (allEqual) ? firstVal : DateTime.MinValue;

                    // Check whether all hidden values are equal
                    allEqual = true;
                    bool firstHidden = m_FileNodes[0].Hidden;
                    foreach (IFileSystemNode node in m_FileNodes)
                    {
                        if (node.Hidden != firstHidden)
                        {
                            allEqual = false;
                        }
                    }
                    if (allEqual)
                    {
                        chkHidden.Checked = firstHidden;
                    }
                    else
                    {
                        chkHidden.CheckState = CheckState.Indeterminate;
                    }
                }
                else
                {
                    lblNumFilesFolders.Visible = false;

                    IFileSystemNode node = m_FileNodes[0];
                    txtName.Text = node.Name;
                    dtpDate.Value = node.ModificationDate;
                    lblSizeValue.Text = FormatFileSize.Format(node.Size);
                    chkHidden.Checked = node.Hidden;

                    if (node is FileNode)
                    {
                        Icon fileIcon = IconReader.GetFileIcon(node.FileSystemPath, IconReader.IconSize.Large, false);
                        picIcon.Image = fileIcon.ToBitmap();
                    }
                    else
                    {
                        Icon fileIcon = IconReader.GetFolderIcon(IconReader.IconSize.Large, IconReader.FolderType.Closed);
                        picIcon.Image = fileIcon.ToBitmap();
                    }
                }

                UpdatePaths();
            }
        }


        #endregion

        public PropertyDialog()
        {
            InitializeComponent();
            AcceptButton = bOK;
            CancelButton = bCancel;
        }

        private void bOK_Click(object sender, EventArgs e)
        {
            IFileSystemNode fileNode = m_FileNodes[0];
            FileLayoutManager manager = fileNode.Parent.Manager;

            // Renaming is only possible for a single node
            if (m_FileNodes.Length == 1)
            {
                if (!manager.Rename(fileNode.FullVirtualPath, txtName.Text))
                {
                    // Message box will be shown
                    DialogResult = DialogResult.None;
                }
            }

            foreach (IFileSystemNode node in m_FileNodes)
            {
                IDataItemX dataItem = manager.GetDataImageItem(node);

                // Set date
                if (dtpDate.Value > DateTime.MinValue)
                {
                    node.ModificationDate = dtpDate.Value;
                    dataItem.SetFileTime(node.ModificationDate);
                }

                // Set hidden
                if (chkHidden.CheckState != CheckState.Indeterminate)
                {
                    node.Hidden = chkHidden.Checked;
                    dataItem.SetIsHidden(node.Hidden);
                }
            }
        }

        private void UpdatePaths()
        {
            // Form already initialised? (OnResize may happen too early)
            if (lblPathValue == null || lblSourceValue == null) return;

            lblPathValue.Text = "-";
            lblSourceValue.Text = "-";

            if (m_FileNodes != null && m_FileNodes.Length == 1)
            {
                lblPathValue.Text = Utility.CompactString(m_FileNodes[0].FullVirtualPath, lblPathValue.Width - 5, lblPathValue.Font, "");
                if (!string.IsNullOrEmpty(m_FileNodes[0].FileSystemPath))
                {
                    lblSourceValue.Text = Utility.CompactString(m_FileNodes[0].FileSystemPath, lblPathValue.Width - 5, lblPathValue.Font, "");
                }
            }
        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);

            UpdatePaths();
        }

    }
}
