﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CDBurnerXP.Forms;
using CDBurnerXP.IO;
using System.Threading;
using System.IO;

namespace CDBurnerXP.Forms
{
    /// <summary>
    /// Represents a dialog which shows the progress while ripping
    /// tracks from an audio disc.
    /// </summary>
    public partial class RipTracksDialog : BaseForm
    {
        private string[] ripPaths;
        private List<string> rippedPaths = new List<string>();
        private bool isGrabbing = false;
        private int currentTrack = 0;
        private string lastFailedTrack = null;

        #region Properties

        protected override bool AutoTranslate
        {
            get { return true; }
        }

        /// <summary>
        /// Returns a list of all ripped tracks.
        /// </summary>
        public string[] RippedPaths
        {
            get
            {
                return this.rippedPaths.ToArray();
            }
        }

        #endregion

        public RipTracksDialog(string[] ripPaths)
        {
            InitializeComponent();

            this.ripPaths = ripPaths;
            AudioConverter.ProgressChanged += new EventHandler<ProgressChangedEventArgs>(AudioConverter_ProgressChanged);
        }

        protected override void OnLoaded(object sender, EventArgs e)
        {
            base.OnLoaded(sender, e);

            RipNextTrack();
        }

        /// <summary>
        /// Rips the next audio track (starting with the first).
        /// </summary>
        private void RipNextTrack()
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke((MethodInvoker)RipNextTrack);
                return;
            }

            if (this.currentTrack > 0 && AudioConverter.IsCancelled)
            {
                // Aborted by user
                this.rippedPaths.Clear();
                this.Close();
                return;
            }

            if (this.lastFailedTrack != null)
            {
                // Warn user, but continue
                string msg = RM.GetString("The track '{0}' cannot be copied, because it is not located on an optical disc. Only add audio tracks (files with the extension CDA) from actual audio discs.");
                MessageBox.Show(string.Format(msg, Path.GetFileName(lastFailedTrack)), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

            if (this.currentTrack >= this.ripPaths.Length)
            {
                // Finished
                this.Close();
                return;
            }

            string currentTrack = this.ripPaths[this.currentTrack];

            // Check whether or not enough space is available, abort 
            if (!DiskSpaceRequiredDialog.Show(this, AudioConverter.GetAudioTrackSize(currentTrack)))
            {
                this.Close();
                return;
            }

            lblStatus.Text = FormatTranslation("Status: Ripping track {0} of {1}...", this.currentTrack + 1, this.ripPaths.Length);

            BeginNextRipTrack(currentTrack);
        }

        /// <summary>
        /// Rips the next track in background.
        /// </summary>
        /// <param name="path">Path to the CDA file that should be ripped</param>
        private void BeginNextRipTrack(string path)
        {
            Thread thread = new Thread(delegate()
            {
                this.isGrabbing = true;

                this.lastFailedTrack = path;

                try
                {
                    string resultFile = AudioConverter.RipAudioTrack(path);
                    if (!string.IsNullOrEmpty(resultFile))
                    {
                        this.rippedPaths.Add(resultFile);
                        this.lastFailedTrack = null;
                    }
                }
                finally
                {
                    this.isGrabbing = false;
                }

                this.currentTrack++;
                RipNextTrack();
            });
            thread.Start();
        }

        private void AudioConverter_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke((MethodInvoker)delegate() { AudioConverter_ProgressChanged(sender, e); });
                return;
            }

            prgRipping.Value = Convert.ToInt32(100.0F * this.currentTrack / this.ripPaths.Length + e.ProgressPercentage / (this.ripPaths.Length));
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);

            if (this.isGrabbing)
            {
                e.Cancel = true;
            }
        }

        protected override void OnClosed(EventArgs e)
        {
            base.OnClosed(e);
            AudioConverter.ProgressChanged -= new EventHandler<ProgressChangedEventArgs>(AudioConverter_ProgressChanged);
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            cmdCancel.Enabled = false;
            lblStatus.Text = RM.GetString("Status: Cancelling ripping, please wait...");
            AudioConverter.CancelRipping();
        }
    }
}
