using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.ComponentModel;
using BassLib = Un4seen.Bass.Bass;
using BassMix = Un4seen.Bass.AddOn.Mix.BassMix;
using BassMisc = Un4seen.Bass.Misc;
using BassTags = Un4seen.Bass.AddOn.Tags;
using BassCd = Un4seen.Bass.AddOn.Cd.BassCd;

namespace CDBurnerXP.IO
{
    public struct ReplayGainInfo
    {
        public double? TrackGain;
        public double? AlbumGain;

        public ReplayGainInfo(double? track, double? album)
        {
            TrackGain = track;
            AlbumGain = album;
        }
    }

    public enum ReplayGainOption
    {
        None,
        Track,
        Album
    }

    public static class AudioConverter
    {
        public static event EventHandler<ProgressChangedEventArgs> ProgressChanged;

        private static string m_TempDir = string.Empty;
        private static ReplayGainOption m_ProcessReplayGain = ReplayGainOption.None;
        private static bool m_CancelRipping = false;
        private static int m_PercentDivisor = 1;
        private static int m_PercentAdd = 0;

        #region Properties

        public static bool IsCancelled
        {
            get
            {
                return m_CancelRipping;
            }
        }

        public static ReplayGainOption ProcessReplayGain
        {
            get
            {
                return m_ProcessReplayGain;
            }
            set
            {
                m_ProcessReplayGain = value;
            }
        }

        public static string TempPath
        {
            set
            {
                m_TempDir = value;
                Directory.CreateDirectory(value);
            }
        }

        #endregion

        public static void CancelRipping()
        {
            m_CancelRipping = true;
        }

        private static void OnProgressChanged(float progress, object item)
        {
            if (ProgressChanged != null)
            {
                progress = (m_PercentAdd + progress) / m_PercentDivisor;
                ProgressChanged(null, new ProgressChangedEventArgs((int)progress, item));
            }
        }

        public static string ToWav(string fileName, int targetSampleRate)
        {
            return ToWav(fileName, targetSampleRate, null)[0];
        }

        /// <summary>
        /// Determines the required size of an audio disc
        /// when ripped to the hard drive.
        /// </summary>
        /// <param name="path">Path to disc</param>
        public static long GetAudioDiscSize(string path)
        {
            long sum = 0;
            string[] files = null;

            try
            {
                files = Directory.GetFiles(path, "*.cda");
            }
            catch (IOException)
            {
                // drive not ready
                return 0L;
            }

            foreach (string file in files)
            {
                sum += GetAudioTrackSize(file);
            }
            return sum;
        }

        /// <summary>
        /// Retrieves the playback length in bytes of a file. 
        /// </summary>
        public static long GetAudioTrackSize(string fileName)
        {
            // reading stream
            int streamIn = BassLib.BASS_StreamCreateFile(fileName, 0, 0, Un4seen.Bass.BASSFlag.BASS_STREAM_PRESCAN);

            if (streamIn == 0)
            {
                // PRESCAN only required for VBR. It seems to fail in rare occasions though.
                streamIn = BassLib.BASS_StreamCreateFile(fileName, 0, 0, Un4seen.Bass.BASSFlag.BASS_STREAM_DECODE);
            }

            if (streamIn == 0) return 0;

            try
            {
                return BassLib.BASS_ChannelGetLength(streamIn);
            }
            finally
            {
                BassLib.BASS_StreamFree(streamIn);
            }
        }

        /// <summary>
        /// Rips all audio tracks from a given disc.
        /// </summary>
        /// <param name="path">Path to disc</param>
        /// <returns>List of all successfully ripped tracks</returns>
        public static string[] RipAllAudioTracks(string path)
        {
            m_CancelRipping = false;
            m_PercentAdd = 0;
            List<string> ripped = new List<string>();

            string[] files = Directory.GetFiles(path, "*.cda");

            if (files.Length == 0) return ripped.ToArray();

            m_PercentDivisor = files.Length;

            try
            {
                foreach (string file in files)
                {
                    string resultFile = RipAudioTrack(file);
                    // Has the process been cancelled? Return nothing in this case.
                    if (m_CancelRipping || string.IsNullOrEmpty(resultFile)) return new string[0];

                    ripped.Add(resultFile);
                    m_PercentAdd += 100;
                }
            }
            finally
            {
                // Reset divisor in any case
                m_PercentDivisor = 1;
                m_PercentAdd = 0;
            }

            return ripped.ToArray();
        }

        /// <summary>
        /// Rips an audio track to hard disc.
        /// </summary>
        /// <returns>File name of the resulting file</returns>
        public static string RipAudioTrack(string fileName)
        {
            m_CancelRipping = false;

            // Open stream. Possibly a special mixed mode file.
            int streamIn = 0;
            if (fileName.StartsWith("MixedMode:"))
            {
                // Determine device index and track index
                fileName = fileName.Replace("MixedMode:", "").Replace(".cda", "");
                string[] indexes = fileName.Split(',');
                if (indexes.Length != 2) return null;
                streamIn = BassCd.BASS_CD_StreamCreate(Conversion.ToInt(indexes[0]), Conversion.ToInt(indexes[1]), Un4seen.Bass.BASSFlag.BASS_STREAM_DECODE);

                // Make human readable file name
                fileName = "Track " + indexes[1] + ".cda";
            }
            else
            {
                streamIn = BassLib.BASS_StreamCreateFile(fileName, 0, 0, Un4seen.Bass.BASSFlag.BASS_STREAM_DECODE);
            }

            if (streamIn == 0) return null;

            string cdTextTitle = Burning.BurningEngine.GetCDtextTitle(fileName);
            string appendToTempName = (cdTextTitle == null) ? Path.GetFileNameWithoutExtension(fileName) : cdTextTitle;

            string targetFile = PathEx.GetTempRipFile(PathEx.ReplaceInvalidFileNameChars(appendToTempName));

            BassMisc.WaveWriter waveFile = new BassMisc.WaveWriter(targetFile, streamIn, true);

            // transfer data from old to new
            long len = BassLib.BASS_ChannelSeconds2Bytes(streamIn, 0.1f);
            long totalBytes = BassLib.BASS_ChannelGetLength(streamIn);
            double totalSeconds = BassLib.BASS_ChannelBytes2Seconds(streamIn, totalBytes);
            short[] data = new short[len / 2 + 1];
            int progress = 0;

            OnProgressChanged(0, fileName);

            try
            {
                while (true)
                {
                    if (m_CancelRipping)
                    {
                        return null;
                    }

                    int getlen = BassLib.BASS_ChannelGetData(streamIn, data, (int)len);
                    if (getlen <= 0) break;

                    double secondsPos = BassLib.BASS_ChannelBytes2Seconds(streamIn, BassLib.BASS_ChannelGetPosition(streamIn));

                    waveFile.Write(data, (int)len);

                    int newProgress = (int)(secondsPos / totalSeconds * 100);
                    if (newProgress != progress)
                    {
                        OnProgressChanged(newProgress, fileName);
                    }
                    progress = newProgress;
                }

                OnProgressChanged(100, fileName);
                return waveFile.FileName;
            }
            finally
            {
                waveFile.Close();
                BassLib.BASS_StreamFree(streamIn);
            }
        }

        /// <summary>
        /// Attempts to convert a given file to WAV, and returns
        /// the result file name (in the system temp-dir).
        /// </summary>
        /// <param name="targetSampleRate">Specify a sample rate > 0 to down- or upsample the file</param>
        /// <returns>null if the conversion failed</returns>
        public static string[] ToWav(string fileName, int targetSampleRate, double[] splitPositions)
        {
            BassMisc.DSP_Gain gain = null;
            List<string> resultFiles = new List<string>();

            // reading stream
            int streamIn = BassLib.BASS_StreamCreateFile(fileName, 0, 0, Un4seen.Bass.BASSFlag.BASS_STREAM_DECODE);
            if (streamIn == 0) return null;

            if (targetSampleRate <= 0) {
                Un4seen.Bass.BASS_CHANNELINFO info = BassLib.BASS_ChannelGetInfo(streamIn);
                targetSampleRate = info.freq;
            }

            int mixerStream = BassMix.BASS_Mixer_StreamCreate(targetSampleRate, 2, Un4seen.Bass.BASSFlag.BASS_STREAM_DECODE);
            BassMix.BASS_Mixer_StreamAddChannel(mixerStream, streamIn, Un4seen.Bass.BASSFlag.BASS_STREAM_DECODE);
            
            if (ProcessReplayGain == ReplayGainOption.Track || ProcessReplayGain == ReplayGainOption.Album) 
            {
                ReplayGainInfo info = GetReplayGainInfo(fileName);
                gain = new BassMisc.DSP_Gain(mixerStream, 10);
                if (ProcessReplayGain == ReplayGainOption.Track)
                {
                    gain.Gain_dBV = Convert.ToDouble((info.TrackGain == null) ? info.AlbumGain : info.TrackGain);
                }
                else
                {
                    gain.Gain_dBV = Convert.ToDouble((info.AlbumGain == null) ? info.TrackGain : info.AlbumGain);
                }
            }

            BassMisc.WaveWriter waveFile = null;

            // transfer data from old to new
            long len = BassLib.BASS_ChannelSeconds2Bytes(mixerStream, 0.1f);
            long totalBytes = BassLib.BASS_ChannelGetLength(streamIn);
            double totalSeconds = BassLib.BASS_ChannelBytes2Seconds(streamIn, totalBytes);
            short[] data = new short[len / 2 + 1];
            int progress = 0;
            int nextSplitIndex = (splitPositions != null && splitPositions.Length > 0) ? 0 : -1;

            OnProgressChanged(0, fileName);
            
            while (true)
            {
                int getlen = BassLib.BASS_ChannelGetData(mixerStream, data, (int)len);
                if (getlen <= 0) break;

                double secondsPos = BassLib.BASS_ChannelBytes2Seconds(mixerStream, BassLib.BASS_ChannelGetPosition(mixerStream));

                // Is splitting enabled?
                if (nextSplitIndex >= 0)
                {
                    // Start with first file as soon as first split position is 
                    // reached (may be 0:00) or next split position is reached.
                    if (splitPositions[nextSplitIndex] <= secondsPos)
                    {
                        if (waveFile != null)
                        {
                            waveFile.Close();
                            resultFiles.Add(waveFile.FileName);
                        }

                        waveFile = new BassMisc.WaveWriter(GetNewTempFile(streamIn, nextSplitIndex + 1), mixerStream, false);

                        nextSplitIndex = (nextSplitIndex < splitPositions.Length - 1) ? ++nextSplitIndex : -1;
                    }
                }
                else if (waveFile == null)
                {
                    // Splitting disabled, create new WAV file once
                    waveFile = new BassMisc.WaveWriter(GetNewTempFile(streamIn, 1), mixerStream, true);
                }

                if (waveFile != null)
                {
                    waveFile.Write(data, (int)len);    
                }

                int newProgress = (int)(secondsPos / totalSeconds * 100);
                if (newProgress != progress)
                {
                    OnProgressChanged(newProgress, fileName);
                }
                progress = newProgress;
            }

            waveFile.Close();
            resultFiles.Add(waveFile.FileName);
            if (gain != null) gain.Stop();

            OnProgressChanged(100, fileName);

            BassLib.BASS_StreamFree(streamIn);
            BassLib.BASS_StreamFree(mixerStream);
            return resultFiles.ToArray();
        }

        public static bool HasReplayGainInfo(string filename)
        {
            ReplayGainInfo info = GetReplayGainInfo(filename);
            return (info.AlbumGain != null || info.TrackGain != null);
        }

        private static string GetNewTempFile(int stream, int part)
        {
            // Prepare a temp location for the new wave file 
            if (stream != 0)
            {
                // Maybe we can find a better file name
                BassTags.TAG_INFO tags = new BassTags.TAG_INFO();
                BassTags.BassTags.BASS_TAG_GetFromFile(stream, tags);
                if (!string.IsNullOrEmpty(tags.title))
                {
                    string name = PathEx.ReplaceInvalidFileNameChars(tags.title) + "-" + part;
                    return PathEx.GetTempRipFile(name);
                }
            }
            
            return PathEx.GetTempRipFile("Track-" + part);
        }

        public static ReplayGainInfo GetReplayGainInfo(string filename)
        {
            BassTags.TAG_INFO info = BassTags.BassTags.BASS_TAG_GetFromFile(filename);
            if (info == null) return new ReplayGainInfo(null, null);

            double? album = null;
            double? track = null;

            foreach (string tag in info.NativeTags)
            {
                string[] keyValue = tag.Split('=');
                if (keyValue.Length != 2) continue;
                if (keyValue[1].Contains(":"))
                {
                    keyValue = keyValue[1].Split(':');
                    if (keyValue.Length != 2) continue;
                }

                string cleanValue = keyValue[1].Trim().TrimEnd('d', 'B');
                if (String.Compare(keyValue[0], "replaygain_track_gain", true) == 0)
                {
                    track = Convert.ToDouble(cleanValue, System.Globalization.CultureInfo.InvariantCulture);
                }
                else if (String.Compare(keyValue[0], "replaygain_album_gain", true) == 0)
                {
                    album = Convert.ToDouble(cleanValue, System.Globalization.CultureInfo.InvariantCulture);
                }
            }

            return new ReplayGainInfo(track, album);
        }
    }
}
