﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Net;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using CDBurnerXP.IO;
using Microsoft.Win32;
using System.Runtime.InteropServices;

namespace CDBurnerXP.Net
{
    /// <summary>
    /// Contains all methods required for the automatic online update.
    /// </summary>
    public class AutoUpdate
    {
        private const string m_UpdateUrl = "http://update.cdburnerxp.se";
        /// <summary>
        /// .org-domain is under ownership of Canneverbe Limited. Just in case.
        /// </summary>
        private const string m_UpdateUrl2 = "http://update.cdburnerxp.org";

        /// <summary>
        /// Gets the folder where all updates are downloaded to.
        /// </summary>
        public static string PathToDownloadedUpdates
        {
            get
            {
                string directory = Path.Combine(Path.GetTempPath(), "CDBurnerXP-updates");
                // Make sure that directory exists
                Directory.CreateDirectory(directory);
                return directory;
            }
        }

#if !UPDATER

        /// <summary>
        /// Gets whether or not online update is possible / allowed.
        /// </summary>
        public static bool IsOnlineUpdateAllowed
        {
            get
            {
                try
                {
                    if (ApplicationSettings.IsPortable())
                    {
                        return false;
                    }

                    IniFile ini = new IniFile("Application", false);

                    bool isMsi = ini.GetBoolValue("Setup", "MsiInstallation", false);
                    bool disableUpdate = ini.GetBoolValue("Setup", "DisableOnlineUpdate", false);

                    return !isMsi && !disableUpdate;
                }
                catch (UnauthorizedAccessException ex)
                {
                    // Common appdata path cannot be accessed
                    CDBurnerXP.IO.Debug.LogError(ex);
                    return false;
                }
            }
        }

        /// <summary>
        /// Opens a website which provides details about available updates.
        /// </summary>
        public static void CheckUpdateOnWebsite()
        {
            try
            {
                string lang = (Thread.CurrentThread.CurrentUICulture == null) ? "" : Thread.CurrentThread.CurrentUICulture.Name + "/";
                Process.Start("http://cdburnerxp.se/" + lang + "download?checkUpdate=" + Application.ProductVersion);
            }
            catch (Exception) { }
        }

        /// <summary>
        /// Starts a thread which checks for new updates and downloads
        /// them if available.
        /// </summary>
        public static void BeginCheckAndDownloadUpdate()
        {
            ThreadPool.QueueUserWorkItem(CheckAndDownloadUpdate, null);
        }

        private static void CheckAndDownloadUpdate(object threadContext)
        {
            try
            {
                ExecuteDownload();
            }
            catch (Exception ex)
            {
                // Never throw exceptions. Will crash the application!
                Console.WriteLine(ex.ToString());
            }
        }

        private static void ExecuteDownload()
        {
            // Only check once a day
            DateTime lastUpdate = Conversion.ToDateTime(Settings.GetValue("LastAutoUpdateCheck", DateTime.MinValue));
            if (lastUpdate >= DateTime.Now.AddDays(-1))
            {
                return;
            }

            Settings.SetValue("LastAutoUpdateCheck", DateTime.Now);

            Version onlineVersion;
            string downloadLocation;

            string customUpdateServer = Settings.GetValue("UpdateServer") as string;
            string updateUrl = customUpdateServer ?? m_UpdateUrl;
            if (Marshal.SizeOf(typeof(IntPtr)) == 8)
            {
                updateUrl += "?x64";
            }

            if (!GetUpdateData(updateUrl, out onlineVersion, out downloadLocation) && !GetUpdateData(m_UpdateUrl2, out onlineVersion, out downloadLocation))
            {
                return;
            }

            // New release available?
            if (onlineVersion > new Version(Application.ProductVersion))
            {
                string fileName = "cdbxp_setup_" + onlineVersion + ".exe";
                fileName = Path.Combine(PathToDownloadedUpdates, fileName);
                if (File.Exists(fileName))
                {
                    // Do not download twice
                    return;
                }

                // Now download file
                string tempFile = Path.GetTempFileName();

                using (WebClient client = new WebClient())
                {
                    client.DownloadFile(downloadLocation, tempFile);
                }

                // Download succeeded, move file to right location
                File.Move(tempFile, fileName);
            }
        }

        private static bool GetUpdateData(string updateUrl, out Version onlineVersion, out string downloadLocation)
        {
            onlineVersion = new Version();
            downloadLocation = "";

            try
            {
                using (WebClient client = new WebClient())
                {
                    client.Headers.Add("User-Agent", Application.ProductName + " " + Application.ProductVersion);
                    string[] updateInfo = client.DownloadString(updateUrl).Split('|');
                    if (updateInfo.Length < 2)
                    {
                        return false;
                    }

                    onlineVersion = new Version(updateInfo[0]);
                    downloadLocation = updateInfo[1];
                }
            }
            catch (Exception)
            {
                return false;
            }
            return true;
        }

#endif

        /// <summary>
        /// Returns the file name to the latest update.
        /// </summary>
        public static string GetLatestUpdate()
        {
            Version latestVersion = null;
            string latestFile = null;

            string[] allFiles = Directory.GetFiles(AutoUpdate.PathToDownloadedUpdates);

            foreach (string file in allFiles)
            {
                try
                {
                    FileVersionInfo versionInfo = FileVersionInfo.GetVersionInfo(file);
                    Version version = new Version(versionInfo.FileVersion);

                    if (latestVersion == null || version > latestVersion)
                    {
                        latestVersion = version;
                        latestFile = file;
                    }
                }
                catch (Exception)
                {
                    // If file cannot be analysed, continue (and delete file)
                    PathEx.TryDeleteFiles(file);
                    continue;
                }
            }

            return latestFile;
        }

        /// <summary>
        /// Escapes multiple shell arguments. Adds quotes and escapes quotes 
        /// within the string.
        /// </summary>
        public static string[] EscapeShellArgument(string[] args)
        {
            string[] escapedArgs = new string[args.Length];

            for (int i = 0; i < args.Length; i++)
            {
                escapedArgs[i] = EscapeShellArgument(args[i]);
            }

            return escapedArgs;
        }

        /// <summary>
        /// Escapes a shell argument. Adds quotes and escapes quotes 
        /// within the string.
        /// </summary>
        public static string EscapeShellArgument(string arg)
        {
            if (arg.Contains(" ") || arg.Contains("\t") || arg.Contains("\""))
            {
                string escapedArg = arg.Replace("\"", "\\\"");
                escapedArg = "\"" + escapedArg + "\"";
                return escapedArg;
            }

            return arg;
        }
    }
}
