using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Net;
using System.Management;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.ComponentModel;
using System.Runtime.InteropServices;
using CDBurnerXP.IO;

namespace CDBurnerXP.Net
{
    public class ErrorReport : IDisposable
    {
        private const string ERROR_URL = "http://www.cdburnerxp.se/error_report.php";

        private static StringBuilder m_DebugInfo = new StringBuilder();

        protected Exception m_Exception = null;
        private bool m_Cancelled = false;
        private string m_ErrorReportText = string.Empty;
        private Bitmap m_Screenshot = null;
        private bool m_SendScreenshot = true;
        private string m_UserInfo = string.Empty;
        private string m_UserEmail = string.Empty;
        private BackgroundWorker m_Worker = null;

        public event AsyncCompletedEventHandler SubmitCompleted;

        #region Properties

        public bool IsBusy
        {
            get
            {
                if (m_Worker == null) return false;
                return m_Worker.IsBusy;
            }
        }

        public string UserInfo
        {
            get
            {
                return m_UserInfo;
            }
            set
            {
                m_UserInfo = value;
            }
        }

        public string UserEmail
        {
            get
            {
                return m_UserEmail;
            }
            set
            {
                m_UserEmail = value;
            }
        }

        public bool SendScreenshot
        {
            get
            {
                return m_SendScreenshot;
            }
            set
            {
                m_SendScreenshot = value;
            }
        }

        public string ErrorReportText
        {
            get {
                if (m_ErrorReportText == string.Empty)
                {
                    GenerateErrorInformation();
                }
                return m_ErrorReportText;
            }
        }

        #endregion

        public ErrorReport(Exception ex)
        {
            m_Exception = ex;
        }

        protected string GetOsType()
        {
            try
            {
                ManagementObjectSearcher searcher = new ManagementObjectSearcher("root\\CIMV2", "SELECT * FROM Win32_Processor");
                foreach (ManagementObject obj in searcher.Get())
                {
                    return (obj["DataWidth"].ToString());
                }
            } catch { }
            return "Unknown";
        }

        protected virtual void GenerateErrorInformation()
        {
            ManagementPath path = new ManagementPath();
            ManagementClass devices = null;
            path.Server = ".";
            path.NamespacePath = @"root\CIMV2";
            path.RelativePath = @"Win32_PnPentity";
            StringBuilder builder = new StringBuilder();

            if (Marshal.SizeOf(typeof(IntPtr)) == 8)
            {
                builder.AppendLine("x64 edition of CDBurnerXP");
            }

            // First, save the exceptions
            builder.Append(m_Exception.ToString() + Environment.NewLine + Environment.NewLine);
            if (m_Exception.InnerException != null)
            {
                builder.Append(m_Exception.InnerException.ToString());
                builder.Append(Environment.NewLine + Environment.NewLine);
            }
            // Save error codes if available
            ExternalException externalEx = m_Exception as ExternalException;
            if (externalEx != null)
            {
                builder.Append("Error code: " + externalEx.ErrorCode.ToString()); 
            }

            builder.Append(Environment.NewLine);
            // Save OS
            builder.Append(Environment.OSVersion.ToString() + " (" + GetOsType() + "bit)" + Environment.NewLine);
            // Save Debug log
            builder.AppendLine("Debug-Log:");
            builder.AppendLine(m_DebugInfo.ToString());

            // Now check some device information
            using (devices = new ManagementClass(new ManagementScope(path), path, new ObjectGetOptions(null, new TimeSpan(0, 0, 0, 2), true)))
            {
                ManagementObjectCollection deviceObjects = devices.GetInstances();
                foreach (ManagementObject device in deviceObjects)
                {
                    PropertyDataCollection properties = device.Properties;
                    string name = device.Properties["Name"].Value as string;
                    builder.Append(name);
                    builder.Append(Environment.NewLine);
                    string description = device.Properties["Description"].Value as string;
                    if (name != description)
                    {
                        builder.Append(description);
                        builder.Append(Environment.NewLine);
                    }
                    builder.Append(Environment.NewLine);
                }
            }

            m_ErrorReportText = builder.ToString();
        }

        public void TakeScreenshot()
        {
            m_Screenshot = new Bitmap(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height, PixelFormat.Format24bppRgb);
            using (Graphics gr = Graphics.FromImage(m_Screenshot)) {
                gr.CopyFromScreen(Screen.PrimaryScreen.Bounds.X, Screen.PrimaryScreen.Bounds.Y, 0, 0, Screen.PrimaryScreen.Bounds.Size, CopyPixelOperation.SourceCopy);
            }
        }

        public void Cancel()
        {
            if (m_Worker != null)
            {
                m_Worker.CancelAsync();
                m_Cancelled = true;
            }
        }

        public void BeginSubmit()
        {
            m_Worker = new BackgroundWorker();
            m_Cancelled = false;
            m_Worker.WorkerSupportsCancellation = true;
            m_Worker.DoWork += new DoWorkEventHandler(m_Worker_DoWork);
            m_Worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(m_Worker_RunWorkerCompleted);
            m_Worker.RunWorkerAsync();
        }

        void m_Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            bool cancelled = e.Cancelled || m_Worker.CancellationPending || m_Cancelled;
            object result = (e.Error == null) ? e.Result : null;
            AsyncCompletedEventArgs args = new AsyncCompletedEventArgs(e.Error, cancelled, result);

            if (SubmitCompleted != null)
            {
                SubmitCompleted(this, args);
            }
        }

        void m_Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                Submit();
            }
            catch (Exception)
            {
                throw;
            }
        }

        public void Submit()
        {
            MultipartUpload uploader = new MultipartUpload(ERROR_URL);
            uploader.FileContentType = "application/octet-stream";
            
            byte[] encBytes = new byte[0];
            uploader.FormFields["userinfo"] = m_UserInfo;
            uploader.FormFields["data"] = ErrorReportText;
            uploader.FormFields["user-email"] = m_UserEmail;
            uploader.FormFields["version"] = Application.ProductVersion;
            uploader.FormFields["exception-type"] = m_Exception.GetType().FullName;

            if (SendScreenshot && m_Screenshot != null)
            {
                // Reduce to 50% JPG, sending should be fast
                EncoderParameters eps = new EncoderParameters(1);
                eps.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 50L);
                ImageCodecInfo ici = GetEncoder(ImageFormat.Jpeg);
                string tmpFile = PathEx.GetTempFileName();
                m_Screenshot.Save(tmpFile, ici, eps);
                uploader.SendFile(tmpFile);
            }
            else
            {
                uploader.SendFile("");
            }
        }

        void UploadDataCompleted(object sender, UploadValuesCompletedEventArgs e)
        {
            if (SubmitCompleted != null)
            {
                SubmitCompleted(this, e);
            }
        }

        private ImageCodecInfo GetEncoder(ImageFormat format)
        {

            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageDecoders();

            foreach (ImageCodecInfo codec in codecs)
            {
                if (codec.FormatID == format.Guid)
                {
                    return codec;
                }
            }
            return null;
        }

        public static void DebugLog(string logLine)
        {
            m_DebugInfo.AppendLine(logLine);
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (m_Worker != null)
            {
                m_Worker.Dispose();
            }
        }

        #endregion
    }
}
