using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.Specialized;
using System.IO;
using System.Net;
using System.Collections;

namespace CDBurnerXP.Net
{
    /// <summary>
    /// Allow the transfer of data files using the W3C's specification
    /// for HTTP multipart form data. Microsoft's version has a bug
    /// where it does not format the ending boundary correctly.
    /// Written by: gregoryp@norvanco.com
    /// </summary>
    public class MultipartUpload
    {
        #region Properties and Members

        private Dictionary<string, string> m_FormFields;
        /// <summary>
        /// Holds any form fields and values that you
        /// wish to transfer with your data.
        /// </summary>
        public Dictionary<string, string> FormFields
        {
            get
            {
                return m_FormFields;
            }
        }
        /// <summary>
        /// Used mainly to avoid passing parameters to other routines.
        /// Could have been local to sendFile().
        /// </summary>
        protected HttpWebRequest m_Request;
        /// <summary>
        /// Difined to build the form field data that is being
        /// passed along with the request.
        /// </summary>
        const string CONTENT_DISP = "Content-Disposition: form-data; name=";
        /// <summary>
        /// Allows you to specify the specific version of HTTP to use for uploads.
        /// The dot NET stuff currently does not allow you to remove the continue-100 header
        /// from 1.1 and 1.0 currently has a bug in it where it adds the continue-100. MS
        /// has sent a patch to remove the continue-100 in HTTP 1.0.
        /// </summary>
        public Version TransferHttpVersion
        {
            get { return m_HttpVersion; } 
            set { m_HttpVersion = value; } 
        }
        Version m_HttpVersion;

        /// <summary>
        /// Used to change the content type of the file being sent.
        /// Currently defaults to: text/xml. Other options are
        /// text/plain or binary
        /// </summary>
        public string FileContentType
        {
            get { return m_FileContentType; }
            set { m_FileContentType = value; }
        }
        string m_FileContentType;

        /// <summary>
        /// Initialize our class for use to send data files.
        /// </summary>
        /// <param name="url">The web address of the recipient of the data transfer.</param>
        public MultipartUpload(string url)
        {
            URL = url;
            m_FormFields = new Dictionary<string, string>();
            m_ResponseText = new StringBuilder();
            BufferSize = 1024 * 10;
            BeginBoundary = "ou812--------------8c405ee4e38917c";
            TransferHttpVersion = HttpVersion.Version11;
            FileContentType = "text/xml";
        }
        //---------- BEGIN PROPERTIES SECTION ----------
        string m_BeginBoundary;
        /// <summary>
        /// The string that defines the begining boundary of
        /// our multipart transfer as defined in the w3c specs.
        /// This method also sets the Content and Ending
        /// boundaries as defined by the w3c specs.
        /// </summary>
        public string BeginBoundary
        {
            get { return m_BeginBoundary; }
            set
            {
                m_BeginBoundary = value;
                ContentBoundary = "--" + BeginBoundary;
                EndingBoundary = ContentBoundary + "--";
            }
        }
        /// <summary>
        /// The string that defines the content boundary of
        /// our multipart transfer as defined in the w3c specs.
        /// </summary>
        protected string ContentBoundary
        {
            get { return m_ContentBoundary; }
            set { m_ContentBoundary = value; }
        }
        string m_ContentBoundary;
        /// <summary>
        /// The string that defines the ending boundary of
        /// our multipart transfer as defined in the w3c specs.
        /// </summary>
        protected string EndingBoundary
        {
            get { return m_EndingBoundary; }
            set { m_EndingBoundary = value; }
        }
        string m_EndingBoundary;
        /// <summary>
        /// The data returned to us after the transfer is completed.
        /// </summary>
        public string ResponseText
        {
            get { return m_ResponseText.ToString(); }
        }
        StringBuilder m_ResponseText;
        /// <summary>
        /// The web address of the recipient of the transfer.
        /// </summary>
        public string URL
        {
            get { return m_URL; }
            set { m_URL = value; }
        }
        string m_URL;
        /// <summary>
        /// Allows us to determine the size of the buffer used
        /// to send a piece of the file at a time out the IO
        /// stream. Defaults to 1024 * 10.
        /// </summary>
        public int BufferSize
        {
            get { return m_BufferSize; }
            set { m_BufferSize = value; }
        }
        int m_BufferSize;
#endregion

        /// <summary>
        /// Here we actually make the request to the web server and
        /// retrieve it's response into a text buffer.
        /// </summary>
        protected virtual void GetResponse()
        {

            System.IO.Stream io;
            WebResponse oResponse;
            try
            {
                oResponse = m_Request.GetResponse();
            }
            catch (WebException web)
            {
                oResponse = web.Response;
            }
            if (null != oResponse)
            {
                io = oResponse.GetResponseStream();
                StreamReader sr = new StreamReader(io);
                string str;
                m_ResponseText.Length = 0;
                while ((str = sr.ReadLine()) != null)
                    m_ResponseText.Append(str);
                oResponse.Close();
            }
            else
                throw new Exception("MultipartForm: Error retrieving server response");
        }
        /// <summary>
        /// Transmits a file to the web server stated in the
        /// URL property. You may call this several times and it
        /// will use the values previously set for fields and URL.
        /// </summary>
        /// <param name="aFilename">The full path of file being transfered.</param>
        public void SendFile(string aFilename)
        {
            // The live of this object is only good during
            // this function. Used mainly to avoid passing
            // around parameters to other functions.
            m_Request = (HttpWebRequest)WebRequest.Create(URL);
            // Set use HTTP 1.0 or 1.1.
            m_Request.ProtocolVersion = TransferHttpVersion;
            m_Request.Method = "POST";
            m_Request.ContentType = "multipart/form-data; boundary=" + BeginBoundary;
            m_Request.Headers.Add("Cache-Control", "no-cache");
            m_Request.KeepAlive = true;
            string strFields = GetFormfields();
            string strFileHdr = GetFileheader(aFilename);
            string strFileTlr = GetFiletrailer();
            m_Request.ContentLength = strFields.Length + strFileTlr.Length;
            if (!string.IsNullOrEmpty(aFilename))
            {
                FileInfo info = new FileInfo(aFilename);
                m_Request.ContentLength += strFileHdr.Length + info.Length;
            }
            System.IO.Stream io;
            io = m_Request.GetRequestStream();
            WriteString(io, strFields);
            if (!string.IsNullOrEmpty(aFilename))
            {
                WriteString(io, strFileHdr);
                this.WriteFile(io, aFilename);
            }
            WriteString(io, strFileTlr);
            GetResponse();
            io.Close();
            // End the life time of this request object.
            m_Request = null;
        }
        /// <summary>
        /// Mainly used to turn the string into a byte buffer and then
        /// write it to our IO stream.
        /// </summary>
        /// <param name="io">The io stream for output.</param>
        /// <param name="str">The data to write.</param>
        protected void WriteString(System.IO.Stream io, string str)
        {
            byte[] PostData = System.Text.Encoding.ASCII.GetBytes(str);
            io.Write(PostData, 0, PostData.Length);
        }
        /// <summary>
        /// Builds the proper format of the multipart data that
        /// contains the form fields and their respective values.
        /// </summary>
        /// <returns>The data to send in the multipart upload.</returns>
        protected string GetFormfields()
        {
            string str = "";
            IDictionaryEnumerator myEnumerator = m_FormFields.GetEnumerator();
            while (myEnumerator.MoveNext())
            {
                str += ContentBoundary + "\r\n" +
                  CONTENT_DISP + '"' + myEnumerator.Key + "\"\r\n\r\n" +
                  myEnumerator.Value + "\r\n";
            }
            return str;
        }
        /// <summary>
        /// Returns the proper content information for the
        /// file we are sending.
        /// </summary>
        /// <remarks>
        /// Hits Patel reported a bug when used with ActiveFile.
        /// Added semicolon after sendfile to resolve that issue.
        /// Tested for compatibility with IIS 5.0 and Java.
        /// </remarks>
        /// <param name="aFilename"></param>
        /// <returns></returns>
        protected string GetFileheader(string aFilename)
        {
            return ContentBoundary + "\r\n" +
              CONTENT_DISP +
              "\"sendfile\"; filename=\"" +
              Path.GetFileName(aFilename) + "\"\r\n" +
              "Content-type: " + FileContentType + "\r\n\r\n";
        }
        /// <summary>
        /// Creates the proper ending boundary for the multipart upload.
        /// </summary>
        /// <returns>The ending boundary.</returns>
        protected string GetFiletrailer()
        {
            return "\r\n" + EndingBoundary;
        }
        /// <summary>
        /// Reads in the file a chunck at a time then sends it to the
        /// output stream.
        /// </summary>
        /// <param name="io">The io stream to write the file to.</param>
        /// <param name="aFilename">The name of the file to transfer.</param>
        protected void WriteFile(System.IO.Stream io, string aFilename)
        {
            FileStream readIn = new FileStream(aFilename, FileMode.Open, FileAccess.Read);
            readIn.Seek(0, SeekOrigin.Begin); // move to the start of the file
            byte[] fileData = new byte[BufferSize];
            int bytes;
            while ((bytes = readIn.Read(fileData, 0, BufferSize)) > 0)
            {
                // read the file data and send a chunk at a time
                io.Write(fileData, 0, bytes);
            }
            readIn.Close();
        }
    }
}