using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Printing;
using System.IO;
using CDBurnerXP.IO;
using CDBurnerXP.Controls;
using Un4seen.Bass.AddOn.Tags;
using BassTags = Un4seen.Bass.AddOn.Tags.BassTags;

namespace CDBurnerXP.Printing
{
    public class CoverPrintDocument : PrintDocument
    {
        #region Enums

        public enum Translations
        {
            Title,
            No,
            Date
        }

        #endregion

        private SizeF DVD_SIZE = new SizeF(1074.8f, 722.47f);
        private SizeF MINI_SIZE = new SizeF(631.92f, 341.58f);
        private SizeF CD_SIZE = new SizeF(952.76f, 470.44f);
        private SizeF CD_BACK_SIZE = new SizeF(590.55f, 460.63f);
        private Font m_HeaderFont = new Font("Tahoma", 8, FontStyle.Bold);

        private CoverType m_CoverType = CoverType.Standard;
        private Dictionary<Translations, string> m_Translation = new Dictionary<Translations, string>();
        private List<CoverListItem> m_FilesAndFolders = new List<CoverListItem>();
        private string m_ImagePath = null;
        private string m_ImageInsidePath = null;
        private bool m_UseAudioTags = false;
        private string m_DisplayFormat = string.Empty;

        private string m_Title = "";
        private int m_Number = 0;
        private DateTime m_Date = DateTime.Now;
        private Image m_Image = null;
        private Image m_ImageInside = null;
        private bool useCoverImage = false;

        #region Properties

        /// <summary>
        /// Gets or sets whether to use a finished (original) cover image as
        /// background, without printing further information on it.
        /// </summary>
        public bool UseCoverImage
        {
            get { return useCoverImage; }
            set { useCoverImage = value; }
        }

        public string DisplayFormat
        {
            get { return m_DisplayFormat; }
            set { m_DisplayFormat = value; }
        }

        public bool UseAudioTags
        {
            get
            {
                return m_UseAudioTags;
            }
            set
            {
                m_UseAudioTags = value;
            }
        }

        public string ImagePath
        {
            get
            {
                return m_ImagePath;
            }
            set
            {
                if (m_ImagePath != value)
                {
                    m_ImagePath = value;
                    if (m_Image != null)
                    {
                        m_Image.Dispose();
                    }
                    m_Image = null;
                }
            }
        }

        public string ImageInsidePath
        {
            get
            {
                return m_ImageInsidePath;
            }
            set
            {
                if (m_ImageInsidePath != value)
                {
                    m_ImageInsidePath = value;
                    if (m_Image != null)
                    {
                        m_Image.Dispose();
                    }
                    m_Image = null;
                }
            }
        }

        protected Image ImageInside
        {
            get
            {
                if (string.IsNullOrEmpty(m_ImageInsidePath)) return null;

                if (m_ImageInside != null) return m_ImageInside;

                try
                {
                    return Bitmap.FromFile(m_ImageInsidePath);
                }
                catch (Exception)
                {
                    return null;
                }
            }
        }

        protected Image Image
        {
            get
            {
                if (string.IsNullOrEmpty(m_ImagePath)) return null;

                if (m_Image != null) return m_Image;

                try
                {
                    return Bitmap.FromFile(m_ImagePath);
                }
                catch (Exception)
                {
                    return null;
                }
            }
        }

        public string Title
        {
            get
            {
                return m_Title;
            }
            set
            {
                m_Title = value;
            }
        }

        public int Number
        {
            get
            {
                return m_Number;
            }
            set
            {
                m_Number = value;
            }
        }

        public DateTime Date
        {
            get
            {
                return m_Date;
            }
            set
            {
                m_Date = value;
            }
        }

        public CoverType CoverType
        {
            get
            {
                return m_CoverType;
            }
            set
            {
                m_CoverType = value;
            }
        }

        public Dictionary<Translations, string> Translation
        {
            get
            {
                return m_Translation;
            }
        }

        public List<CoverListItem> FilesAndFolders
        {
            get
            {
                return m_FilesAndFolders;
            }
        }

        protected RectangleF BoundingRectangle
        {
            get
            {
                SizeF size;

                switch (CoverType) {
                    case CoverType.DVD:
                        size = DVD_SIZE;
                        break;
                    case CoverType.MiniDisc:
                        size = MINI_SIZE;
                        break;
                    case CoverType.StandardBack:
                        size = CD_BACK_SIZE;
                        break;
                    case CoverType.Standard:
                    case CoverType.SlimCD:
                    default:
                        size = CD_SIZE;
                        break;
                }

                float x = (this.DefaultPageSettings.Bounds.Width - size.Width) / 2f;
                x -= DefaultPageSettings.HardMarginX;
                float y = (this.DefaultPageSettings.Bounds.Height - size.Height) / 2f;
                y -= DefaultPageSettings.HardMarginY;
                return new RectangleF(x, y, size.Width, size.Height);
            }
        }

        #endregion

        public CoverPrintDocument()
            : base()
        {
            Translation[Translations.Date] = "Date: ";
            Translation[Translations.No] = "No.: ";
            Translation[Translations.Title] = "Title: ";

            this.DefaultPageSettings.Landscape = true;
            this.OriginAtMargins = false;
        }

        protected override void OnPrintPage(PrintPageEventArgs e)
        {
            base.OnPrintPage(e);

            e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            switch (CoverType)
            {
                case CoverType.DVD:
                    DrawDvdCase(e.Graphics);
                    break;
                case CoverType.Standard:
                case CoverType.SlimCD:
                    DrawStandardCase(e.Graphics);
                    break;
                case CoverType.StandardBack:
                    DrawStandardCaseBack(e.Graphics);
                    break;
                case CoverType.MiniDisc:
                    DrawMiniCase(e.Graphics);
                    break;
            }
        }

        protected override void OnQueryPageSettings(QueryPageSettingsEventArgs e)
        {
            base.OnQueryPageSettings(e);
            e.PageSettings.Landscape = true;
        }

        #region Print functions

        private void DrawDvdCase(Graphics g)
        {
            if (this.useCoverImage && Image != null)
            {
                g.DrawImage(Image, BoundingRectangle);
            }
            else
            {
                Font dvdFont = new Font("Tahoma", 12, FontStyle.Bold);
                Pen pGray = new Pen(Color.Gray, 0.8F);
                Pen pLightGray = new Pen(Color.LightGray, 0.8F);

                // Determine the surrounding rectangle
                RectangleF rect = BoundingRectangle;
                g.DrawRectangle(pGray, rect.X, rect.Y, rect.Width, rect.Height);

                // Draw header
                g.DrawString(Translation[Translations.Title] + Title, m_HeaderFont, Brushes.Black, rect.X + 3.82f, rect.Y + 7.82f);
                if (Number > 0)
                {
                    g.DrawString(Translation[Translations.No] + Number.ToString(), m_HeaderFont, Brushes.Black, rect.X + 448.82f, rect.Y + 7.82f);
                }
                g.DrawString(Translation[Translations.Date] + Date.ToShortDateString(), m_HeaderFont, Brushes.Black, rect.X + 3.82f, rect.Y + 26.82f);
                g.DrawLine(pGray, rect.X + 5.82f, rect.Y + 42.82f, rect.X + 502.82f, rect.Y + 42.82f);

                // Draw back
                g.DrawRectangle(pGray, rect.X + 509.87f, rect.Y, 55.12f, 722.47f);
                g.RotateTransform(90f);
                if (Number > 0)
                {
                    g.DrawString(Number.ToString(), dvdFont, Brushes.Black, rect.Y + 10f, -rect.Right + rect.Width / 2f - 10f);
                }
                g.DrawString(Title, dvdFont, Brushes.Black, rect.Y + 50f, -rect.Right + rect.Width / 2f - 10f);
                g.DrawString(Date.ToShortDateString(), dvdFont, Brushes.Black, rect.Y + 600f, -rect.Right + rect.Width / 2f - 10f);
                g.ResetTransform();

                // Draw front
                float titleHeight = g.MeasureString(Title, dvdFont).Height;
                RectangleF titleBounds = new RectangleF(rect.X + 578.82f, rect.Y + 527.64f, 483f, titleHeight);
                g.DrawRectangle(pLightGray, titleBounds.X, titleBounds.Y, titleBounds.Width, titleBounds.Height);

                StringFormat format = new StringFormat();
                format.Alignment = StringAlignment.Center;
                format.FormatFlags = StringFormatFlags.NoWrap;
                g.DrawString(Title, dvdFont, Brushes.Black, titleBounds, format);

                Rectangle imageBounds = new Rectangle((int)rect.X + 579, (int)rect.Y + 29, 476, 470);
                if (Image != null)
                {
                    g.DrawImage(Image, imageBounds);
                }

                pGray.Dispose();
                pLightGray.Dispose();

                DrawCaseInfo(g, 6.82f, 45.82f, 410f, 0, 45);
            }
        }

        private void DrawCaseInfo(Graphics g, float leftOffset, float topOffset, float fileInfoWidth, int startIndex, int maxItems)
        {
            Font stdFont = new Font("Tahoma", 8, FontStyle.Regular);

            float rowHeight = g.MeasureString("0", stdFont).Height;
            StringFormat format = new StringFormat();
            format.FormatFlags = StringFormatFlags.NoWrap;
            RectangleF rect = BoundingRectangle;

            int showItemsCount = Math.Min(maxItems, m_FilesAndFolders.Count - startIndex);

            RectangleF rowBounds = new RectangleF(rect.X + leftOffset, rect.Y + topOffset, 20f, rowHeight);

            for (int i = startIndex; i < startIndex + showItemsCount; i++)
            {
                // Just in case. Shouldn't happen, but there once was a problem
                // with new folders and a "null" path.
                if (string.IsNullOrEmpty(m_FilesAndFolders[i].DisplayName)) continue;

                g.DrawString((i+1).ToString(), stdFont, Brushes.Black, rowBounds, format);
                rowBounds.X += 22;

                rowBounds.Width = fileInfoWidth;
                TAG_INFO tagInfo = null;

                string defaultFileNameString = m_FilesAndFolders[i].DisplayName;
                if (UseAudioTags)
                {
                    // For audio covers, it certainly makes sense to use the artist/album/title info
                    // instead of the plain file name
                    tagInfo = BassTags.BASS_TAG_GetFromFile(m_FilesAndFolders[i].FullPath);
                    string desc = AudioTrack.FormatAudioTags(tagInfo);
                    if (!string.IsNullOrEmpty(desc))
                    {
                        defaultFileNameString = desc;
                    }
                }

                string formattedFileName = m_DisplayFormat;
                // Replace variables with data
                if (!string.IsNullOrEmpty(m_DisplayFormat))
                {
                    formattedFileName = formattedFileName.Replace("{default}", defaultFileNameString);
                    formattedFileName = formattedFileName.Replace("{filename}", defaultFileNameString);
                    formattedFileName = formattedFileName.Replace("{filenameWithoutExtension}", Path.GetFileNameWithoutExtension(m_FilesAndFolders[i].DisplayName));
                    formattedFileName = formattedFileName.Replace("{fullPath}", m_FilesAndFolders[i].FullPath);
                    formattedFileName = formattedFileName.Replace("{extension}", Path.GetExtension(m_FilesAndFolders[i].DisplayName).TrimStart('.'));
                    if (tagInfo != null)
                    {
                        formattedFileName = formattedFileName.Replace("{album}", tagInfo.album);
                        if (m_FilesAndFolders[i].Track != null && !string.IsNullOrEmpty(m_FilesAndFolders[i].Track.CDTextTitle))
                        {
                            formattedFileName = formattedFileName.Replace("{title}", m_FilesAndFolders[i].Track.CDTextTitle);
                        }
                        else
                        {
                            formattedFileName = formattedFileName.Replace("{title}", tagInfo.title);
                        }
                        if (m_FilesAndFolders[i].Track != null && !string.IsNullOrEmpty(m_FilesAndFolders[i].Track.CDTextArtist))
                        {
                            formattedFileName = formattedFileName.Replace("{artist}", m_FilesAndFolders[i].Track.CDTextArtist);
                        }
                        else
                        {
                            formattedFileName = formattedFileName.Replace("{artist}", tagInfo.artist);
                        }

                        formattedFileName = formattedFileName.Replace("{trackNo}", tagInfo.track);
                        formattedFileName = formattedFileName.Replace("{genre}", tagInfo.genre);
                    }
                }

                defaultFileNameString = string.IsNullOrEmpty(formattedFileName) ? defaultFileNameString : formattedFileName;

                g.DrawString(defaultFileNameString, stdFont, Brushes.Black, rowBounds, format);
                rowBounds.X += fileInfoWidth + 2;

                rowBounds.Width = 60;
                format.Alignment = StringAlignment.Far;
                g.DrawString(m_FilesAndFolders[i].AdditionalInfo, stdFont, Brushes.Black, rowBounds, format);
                
                format.Alignment = StringAlignment.Near;
                rowBounds.Y += rowHeight;
                rowBounds.X = rect.X + leftOffset;
            }

        }

        private void DrawStandardCaseBack(Graphics g)
        {
            if (this.useCoverImage && Image != null)
            {
                g.DrawImage(Image, BoundingRectangle);
            }
            else
            {
                Pen pGray = new Pen(Color.Gray, 0.8f);

                // Determine the surrounding rectangles
                RectangleF rect = BoundingRectangle;
                //287.4f, 157.48f
                g.DrawRectangle(pGray, rect.X, rect.Y, rect.Width, rect.Height);
                g.DrawRectangle(pGray, rect.X + 23.62f, rect.Y, 543.31f, 460.63f);

                // Draw left edge
                g.RotateTransform(90f);
                if (Number > 0)
                {
                    g.DrawString(Number.ToString(), m_HeaderFont, Brushes.Black, rect.Y + 5f, -rect.Right + rect.Width - 20f);
                }
                g.DrawString(Title, m_HeaderFont, Brushes.Black, rect.Y + 50f, -rect.Right + rect.Width - 20f);
                g.DrawString(Date.ToShortDateString(), m_HeaderFont, Brushes.Black, rect.Y + 380f, -rect.Right + rect.Width - 20f);
                // Right side
                if (Number > 0)
                {
                    g.DrawString(Number.ToString(), m_HeaderFont, Brushes.Black, rect.Y + 5f, -rect.Right + 5f);
                }
                g.DrawString(Title, m_HeaderFont, Brushes.Black, rect.Y + 50f, -rect.Right + 5f);
                g.DrawString(Date.ToShortDateString(), m_HeaderFont, Brushes.Black, rect.Y + 380f, -rect.Right + 5f);
                g.ResetTransform();

                pGray.Dispose();

                DrawCaseInfo(g, 30f, 8f, 445f, 0, 30);
            }
        }

        private void DrawMiniCase(Graphics g)
        {
            // Determine the surrounding rectangle
            RectangleF rect = BoundingRectangle;

            if (this.useCoverImage)
            {
                if (Image != null)
                {
                    g.DrawImage(Image, new RectangleF(rect.X + rect.Width / 2f, rect.Y, rect.Width / 2f + 2, rect.Bottom - rect.Y + 1));
                }
                if (ImageInside != null)
                {
                    g.DrawImage(ImageInside, new RectangleF(rect.X, rect.Y, rect.Width / 2f, rect.Bottom - rect.Y + 1));
                }
            }
            else
            {
                Pen pPen = new Pen(Color.Gray, 0.8f);

                g.DrawRectangle(pPen, rect.X, rect.Y, rect.Width, rect.Height);

                // Draw header
                g.DrawString(Translation[Translations.Title] + Title, m_HeaderFont, Brushes.Black, rect.X + 7.09f, rect.Y + 6.78f);
                if (Number > 0)
                {
                    g.DrawString(Translation[Translations.No] + Number, m_HeaderFont, Brushes.Black, rect.X + 260f, rect.Y + 6.78f);
                }
                g.DrawString(Translation[Translations.Date] + Date.ToShortDateString(), m_HeaderFont, Brushes.Black, rect.X + 7.09f, rect.Y + 21.78f);
                g.DrawLine(pPen, rect.X + 8.09f, rect.Y + 35.78f, rect.X + 305.09f, rect.Y + 35.78f);

                // Draw Image
                g.DrawLine(pPen, rect.X + rect.Width / 2f, rect.Y, rect.X + rect.Width / 2f, rect.Y + rect.Height);
                if (Image != null)
                {
                    g.DrawImage(Image, new RectangleF(rect.X + rect.Width / 2f, rect.Y, rect.Width / 2, rect.Height));
                }

                pPen.Dispose();

                DrawCaseInfo(g, 10f, 40f, 210f, 0, 20);
            }
        }

        private void DrawStandardCase(Graphics g)
        {
            // Determine the surrounding rectangle
            RectangleF rect = BoundingRectangle;

            Pen pGray = new Pen(Color.Gray, 0.8f);

            Image imgLoaded = Image;
            Image imgInsideLoaded = ImageInside;

            if (this.useCoverImage)
            {
                if (imgLoaded != null)
                {
                    g.DrawImage(imgLoaded, new RectangleF(rect.X + rect.Width / 2f, rect.Y, rect.Width / 2f + 2, rect.Bottom - rect.Y + 1));
                }
                if (imgInsideLoaded != null)
                {
                    g.DrawImage(imgInsideLoaded, new RectangleF(rect.X, rect.Y, rect.Width / 2f, rect.Bottom - rect.Y + 1));
                }

                g.DrawRectangle(pGray, rect.X, rect.Y, rect.Width, rect.Height);
            }
            else
            {
                g.DrawRectangle(pGray, rect.X, rect.Y, rect.Width, rect.Height);

                //Left side
                g.DrawString(Translation[Translations.Title] + Title, m_HeaderFont, Brushes.Black, rect.X + 11.81f, rect.Y + 11.81f);
                if (Number > 0)
                {
                    g.DrawString(Translation[Translations.No] + Number, m_HeaderFont, Brushes.Black, rect.X + 411.95f, rect.Y + 11.81f);
                }
                g.DrawString(Translation[Translations.Date] + Date.ToShortDateString(), m_HeaderFont, Brushes.Black, rect.X + 11.81f, rect.Y + 31.49f);
                g.DrawLine(pGray, rect.X + 11.81f, rect.Y + 46.49f, rect.X + 461.81f, rect.Y + 46.49f);

                //Right side
                g.DrawLine(pGray, rect.X + 476.38f, rect.Y, rect.X + 476.38f, rect.Bottom);
                pGray.Dispose();

                RectangleF imageRect = new RectangleF(rect.X + 476.38f, rect.Y, rect.Width / 2f + 2, rect.Bottom - rect.Y + 1);

                if (imgLoaded != null)
                {
                    g.DrawImage(imgLoaded, imageRect);
                    DrawCaseInfo(g, 15f, 50f, 360f, 0, 28);
                }
                else
                {
                    // If no image is used, use both sides for tracks
                    if (m_FilesAndFolders.Count > 28)
                    {
                        // Draw on front starting with track 1
                        DrawCaseInfo(g, rect.X + 400f, 8f, 360f, 0, 31);
                        // and draw rest on back.
                        DrawCaseInfo(g, 15f, 50f, 360f, 31, 28);
                    }
                    else
                    {
                        // Draw on back
                        DrawCaseInfo(g, 15f, 50f, 360f, 0, 28);
                    }
                }
            }
        }

        #endregion
    }
}
