using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace CDBurnerXP.Windows.Forms
{
  partial class CommandButton : Button
  {
    enum eButtonState { Normal, MouseOver, Down }
    eButtonState m_State = eButtonState.Normal;

    Image imgArrow1 = null;
    Image imgArrow2 = null;

    // make sure the control is invalidated(repainted) when the text is changed
    public override string Text { get { return base.Text; } set { base.Text = value; this.Invalidate(); } }

    Font m_smallFont;
    public Font SmallFont { get { return m_smallFont; } set { m_smallFont = value; } }

    bool m_autoHeight = true;
    public bool AutoHeight { get { return m_autoHeight; } set { m_autoHeight = value; if (m_autoHeight) this.Invalidate(); } }

    //--------------------------------------------------------------------------------
    public CommandButton()
    {
      InitializeComponent();
      base.Font = new Font("Arial", 11.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (byte)0);
      m_smallFont = new Font("Arial", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (byte)0);
    }

    //--------------------------------------------------------------------------------
    protected override void OnCreateControl()
    {
      base.OnCreateControl();
      imgArrow1 = CsLib.Properties.Resources.green_arrow1;
      imgArrow2 = CsLib.Properties.Resources.green_arrow2;
    }

    //--------------------------------------------------------------------------------
    const int LEFT_MARGIN = 10;
    const int TOP_MARGIN = 10;
    const int ARROW_WIDTH = 15;

    string GetLargeText()
    {
      string[] lines = this.Text.Split(new char[] { '\n' });
      return lines[0];
    }

    string GetSmallText()
    {
      if (this.Text.IndexOf('\n') < 0)
        return "";
      
      string s = this.Text;
      string[] lines = s.Split(new char[] { '\n' });
      s = "";
      for (int i = 1; i < lines.Length; i++)
        s += lines[i] + "\n";
      return s.Trim(new char[] { '\n' });
    }

    Size GetLargeTextSize()
    {
      int x = LEFT_MARGIN + ARROW_WIDTH + 5;
      Size mzSize = new Size(this.Width - x - LEFT_MARGIN, 5000);  // presume RIGHT_MARGIN = LEFT_MARGIN
      Size textSize;
      using (Graphics g = this.CreateGraphics())
      {
          textSize = TextRenderer.MeasureText(g, GetLargeText(), base.Font, mzSize, TextFormatFlags.SingleLine | TextFormatFlags.Left);
      }
      textSize.Width = mzSize.Width;
      return textSize;
    }

    Size GetSmallTextSize()
    {
      string s = GetSmallText();
      if (s == "") return new Size(0, 0);
      int x = LEFT_MARGIN + ARROW_WIDTH + 5; // <- indent small text slightly more
      Size mzSize = new Size(this.Width - x - LEFT_MARGIN, 5000);  // presume RIGHT_MARGIN = LEFT_MARGIN
      Size textSize;
      using (Graphics g = this.CreateGraphics())
      {
          textSize = TextRenderer.MeasureText(g, s + "        ", m_smallFont, mzSize, TextFormatFlags.WordBreak);
      }
      if (mzSize.Width > textSize.Width)
      {
          textSize.Width = mzSize.Width;
      }

      return textSize;
    }

    public int GetBestHeight()
    {
      //return 40;
      return (TOP_MARGIN * 2) + (int)GetSmallTextSize().Height + (int)GetLargeTextSize().Height;
    }

    //--------------------------------------------------------------------------------
    protected override void OnPaint(PaintEventArgs e)
    {
      e.Graphics.SmoothingMode = SmoothingMode.HighQuality;
      e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;

      LinearGradientBrush brush;
      LinearGradientMode mode = LinearGradientMode.Vertical;

      Rectangle newRect = new Rectangle(ClientRectangle.X, ClientRectangle.Y, ClientRectangle.Width - 1, ClientRectangle.Height - 1);
      Color text_color = SystemColors.WindowText;

      Image img = imgArrow1;

      if (Enabled)
      {
        switch (m_State)
        {
          case eButtonState.Normal:
            e.Graphics.FillRectangle(SystemBrushes.Window, newRect);
            if (base.Focused)
              e.Graphics.DrawRectangle(new Pen(Color.SkyBlue, 1), newRect);
            else
                e.Graphics.DrawRectangle(new Pen(SystemColors.Window, 1), newRect);
            text_color = Color.DarkBlue;
            break;

          case eButtonState.MouseOver:
            brush = new LinearGradientBrush(newRect, SystemColors.Window, ControlPaint.Light(SystemColors.Control), mode);
            e.Graphics.FillRectangle(brush, newRect);
            e.Graphics.DrawRectangle(new Pen(Color.Silver, 1), newRect);
            img = imgArrow2;
            text_color = Color.Blue;
            break;

          case eButtonState.Down:
            brush = new LinearGradientBrush(newRect, ControlPaint.Light(SystemColors.Control), SystemColors.Window, mode);
            e.Graphics.FillRectangle(brush, newRect);
            e.Graphics.DrawRectangle(new Pen(Color.DarkGray, 1), newRect);
            text_color = Color.DarkBlue;
            break;
        }
      }
      else
      {
          brush = new LinearGradientBrush(newRect, SystemColors.Control, ControlPaint.Dark(SystemColors.Control), mode);
        e.Graphics.FillRectangle(brush, newRect);
        e.Graphics.DrawRectangle(new Pen(Color.DarkGray, 1), newRect);
        text_color = Color.DarkBlue;
      }


      string largetext = this.GetLargeText();
      string smalltext = this.GetSmallText();

      Size szL = GetLargeTextSize();
      int xPos = LEFT_MARGIN + ARROW_WIDTH + 5;
      TextRenderer.DrawText(e.Graphics, largetext, base.Font, new Rectangle(new Point(xPos, TOP_MARGIN), szL), text_color, TextFormatFlags.Left); 

      if (smalltext != "")
      {
        Size szS = GetSmallTextSize();
        Rectangle rect = new Rectangle(new Point(xPos, TOP_MARGIN + (int)szL.Height), szS);
        TextRenderer.DrawText(e.Graphics, smalltext, m_smallFont, rect, text_color, TextFormatFlags.WordBreak | TextFormatFlags.NoClipping);
      }

      e.Graphics.DrawImage(img, new Point(LEFT_MARGIN, TOP_MARGIN + (int)(szL.Height/2) - (int)(img.Height/2)));
    }

    //--------------------------------------------------------------------------------
    protected override void OnMouseLeave(System.EventArgs e)
    {
      m_State = eButtonState.Normal;
      this.Invalidate();
      base.OnMouseLeave(e);
    }

    //--------------------------------------------------------------------------------
    protected override void OnMouseEnter(System.EventArgs e)
    {
      m_State = eButtonState.MouseOver;
      this.Invalidate();
      base.OnMouseEnter(e);
    }

    //--------------------------------------------------------------------------------
    protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
    {
      m_State = eButtonState.MouseOver;
      this.Invalidate();
      base.OnMouseUp(e);
    }

    //--------------------------------------------------------------------------------
    protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
    {
      m_State = eButtonState.Down;
      this.Invalidate();
      base.OnMouseDown(e);
    }

    //--------------------------------------------------------------------------------
    protected override void OnSizeChanged(EventArgs e)
    {
      if (m_autoHeight)
      {
        int h = GetBestHeight();
        if (this.Height != h)
        {
          this.Height = h;
          return;
        }
      }
      base.OnSizeChanged(e);
    }

    //--------------------------------------------------------------------------------
  }
}
